from PyQt5.QtCore import Qt,  pyqtSlot
from PyQt5.QtGui import QKeySequence
from PyQt5.QtWidgets import QApplication, QHeaderView, QTableWidget, QFileDialog,  QTableWidgetItem
from xulpymoney.libxulpymoney import Money
from xulpymoney.libxulpymoneyfunctions import qright, qleft
from officegenerator import ODS_Write, Currency, Percentage,  Coord
import datetime
import logging
from decimal import Decimal

class myQTableWidget(QTableWidget):
    def __init__(self, parent):
        QTableWidget.__init__(self, parent)
        self.parent=parent
        self.mem=None
        self.sectionname=None
        self._save_settings=True
        self.setAlternatingRowColors(True)
        self.saved_printed=False#To avoid printing a lot of times
        self._last_height=None
        
        
    def setVerticalHeaderHeight(self, height):
        """height, if null default.
        Must be after settings"""
        if height==None:
            self.verticalHeader().setSectionResizeMode(QHeaderView.ResizeToContents)
            self._last_height=None
        else:
            self.verticalHeader().setSectionResizeMode(QHeaderView.Fixed)
            self.verticalHeader().setDefaultSectionSize(height) 
            self._last_height=height

    def setSaveSettings(self, state):
        """Used when i don't want my columns with being saved"""
        self._save_settings=state

    def sectionResized(self, logicalIndex, oldSize, newSize):
        modifiers = QApplication.keyboardModifiers()
        if modifiers == Qt.ShiftModifier:
            for i in range(self.columnCount()):
                self.setColumnWidth(i, newSize)
        elif modifiers == Qt.ControlModifier:
            self.resizeRowsToContents()
            self.resizeColumnsToContents()
        self.save()
            
            
    def save(self):
        if self._save_settings==True:
            self.mem.settings.setValue("{}/{}_horizontalheader_state".format(self.sectionname, self.objectName()), self.horizontalHeader().saveState() )
            if self.saved_printed==False: 
                print("Saved {}/{}_horizontalheader_state".format(self.sectionname, self.objectName()))
                self.saved_printed=True
        
    def settings(self, mem, sectionname,  objectname=None):
        """objectname used for dinamic tables"""
        self.mem=mem
        self.setVerticalHeaderHeight(int(self.mem.settings.value("myQTableWidget/rowheight", 24)))
        self.sectionname=sectionname
        if objectname!=None:
            self.setObjectName(objectname)

    def applySettings(self):
        """settings must be defined before"""
        self.horizontalHeader().setSectionResizeMode(QHeaderView.Interactive)
        self.horizontalHeader().sectionResized.connect(self.sectionResized)
        state=self.mem.settings.value("{}/{}_horizontalheader_state".format(self.sectionname, self.objectName()))
        if state:
            self.horizontalHeader().restoreState(state)
        

    def clear(self):
        """Clear table"""
        self.setRowCount(0)
        self.clearContents()

    def verticalScrollbarAction(self,  action):
        """Resizes columns if column width is less than table hint"""
        for i in range(self.columnCount()):
            if self.sizeHintForColumn(i)>self.columnWidth(i):
                self.setColumnWidth(i, self.sizeHintForColumn(i))

    @pyqtSlot()
    def keyPressEvent(self, event):
        if  event.matches(QKeySequence.ZoomIn) and self._last_height!=None:
            height=int(self.mem.settings.value("myQTableWidget/rowheight", 24))
            self.mem.settings.setValue("myQTableWidget/rowheight", height+1)
            logging.info("Setting myQTableWidget/rowheight set to {}".format(self.mem.settings.value("myQTableWidget/rowheight", 24)))
            self.setVerticalHeaderHeight(int(self.mem.settings.value("myQTableWidget/rowheight", 24)))
        elif  event.matches(QKeySequence.ZoomOut) and self._last_height!=None:
            height=int(self.mem.settings.value("myQTableWidget/rowheight", 24))
            self.mem.settings.setValue("myQTableWidget/rowheight", height-1)
            ("Setting myQTableWidget/rowheight set to {}".format(self.mem.settings.value("myQTableWidget/rowheight", 24)))
            self.setVerticalHeaderHeight(int(self.mem.settings.value("myQTableWidget/rowheight", 24)))
        elif event.matches(QKeySequence.Print):
            filename = QFileDialog.getSaveFileName(self, self.tr("Save File"), "table.ods", self.tr("Libreoffice calc (*.ods)"))[0]
            if filename:
                Table2ODS(self.mem,filename, self, "My table")

    ## Adds a horizontal header array , a vertical header array and a data array
    ##
    ## Automatically set alignment
    def setData(self, header_horizontal, header_vertical, data):
        self.data_header_horizontal=header_horizontal
        self.data_header_vertical=header_vertical
        self.data=data
        self.setColumnCount(len(self.data_header_horizontal))
        for i in range(len(self.data)):
            self.setHorizontalHeaderItem(i, QTableWidgetItem(self.data_header_horizontal[i]))
        #DATA  
        self.applySettings()
        self.clearContents()
        
        self.setRowCount(self.length()+1)
        
        self.applySettings()
        self.setRowCount(self.length())        
        for row in len(self.data):
            for column in len(self.data_header_horizontal) :
                self.setitem(row, column, self.object2qtablewidgetitem(self.data[row][column]))
                    
    ## Converts a objecct class to a qtablewidgetitem
    def object2qtablewidgetitem(self, o):
        if o.__class__ in [int,  float, Decimal, Money]:
            return qright(o)
        else:
            return qleft(o)
    ## Converts self.data to an other list with officegenerator objects
    def data2officegeneratordata(self, arr):
        r=[]
        for i,  row in enumerate(self.arr):
            for j,  column in enumerate(row):
                r.append(self.data[row][column])
            
            
class Table2ODS(ODS_Write):
    def __init__(self, mem, filename, table, title):
        ODS_Write.__init__(self, filename)
        self.mem=mem
        sheet=self.createSheet(title)
        #Array width
        widths=[]
        if not table.verticalHeader().isHidden():
            widths.append(table.verticalHeader().width()*0.90)
        for i in range(table.columnCount()):
            widths.append(table.columnWidth(i)*0.90)
        sheet.setColumnsWidth(widths)

        #firstcontentletter and firstcontentnumber
        if table.horizontalHeader().isHidden() and not table.verticalHeader().isHidden():
            coord=Coord("B1")
        elif not table.horizontalHeader().isHidden() and table.verticalHeader().isHidden():
            coord=Coord("A2")
        elif not table.horizontalHeader().isHidden() and not table.verticalHeader().isHidden():
            coord=Coord("B2")
        elif table.horizontalHeader().isHidden() and table.verticalHeader().isHidden():
            coord=Coord("A1")
        sheet.setSplitPosition(coord)
        #HH
        if not table.horizontalHeader().isHidden():
            for letter in range(table.columnCount()):
                sheet.add(Coord(coord.letter + "1").addColumn(letter), table.horizontalHeaderItem(letter).text(), "OrangeCenter")
        logging.debug("HH Done")
        #VH
        if not table.verticalHeader().isHidden():
            for number in range(table.rowCount()):
                try:#Caputuro cuando se numera sin items 1, 2, 3
                    sheet.add(Coord("A" + coord.number).addRow(number), table.verticalHeaderItem(number).text(), "YellowLeft")
                except:
                    pass
        logging.debug("VH Done")
        #Items
        for number in range(table.rowCount()):
            for letter in range(table.columnCount()):
                try:
                    o=self.itemtext2object(table.item(number, letter).text())
                    sheet.add(Coord(coord.string()).addColumn(letter).addRow(number),o, self.object2style(o))
                except:#Not a QTableWidgetItem or NOne
                    pass
        logging.debug("Items done")
        sheet.setCursorPosition(coord.letter+ str(table.rowCount()+2))
        self.save()

    def itemtext2object(self, t):
        """
            Convierte t en un Money, Percentage o lo deja como text
        """
        if t[-2:]==" %":
            try:
                number=Decimal(t.replace(" %", ""))
                return Percentage(number, 100)
            except:
                logging.info("Error converting percentage")
                pass
        elif t[-2:] in (" €"," $"):
           try:
                number=Decimal(t.replace(t[-2:], "").replace(".", "").replace(",", "."))
                return Currency(number, self.mem.currencies.find_by_symbol(t[-1:]).id)
           except:
                logging.info("Error converting Money")
        elif t.find(":")!=-1 and t.find("-")!=-1:
            try:
                return datetime.datetime.strptime(t, "%Y-%m-%d %H:%M:%S")
            except:
                logging.info("Error convertir datetime {}".format(t))
        elif t.find("-")!=-1:
            try:
                return datetime.datetime.strptime(t, "%Y-%m-%d").date()
            except:
                logging.info("Error convertir date {}".format(t))
        elif t.find(".")!=-1:
            try:
                return Decimal(t)
            except:
                logging.info("Error convertir Decimal {}".format(t))
        else:
            try:
                return int(t)
            except:
                logging.info("Error convertir Integer {}".format(t))
        return t


    def object2style(self, o):
        """
            Define el style de un objeto
        """
        if o.__class__==Currency:
            return "WhiteEuro"
        elif o.__class__==Percentage:
            return "WhitePercentage"
        elif o.__class__==datetime.datetime:
            return "WhiteDatetime"
        elif o.__class__==datetime.date:
            return "WhiteDate"
        elif o.__class__==Decimal:
            return "WhiteDecimal6"
        elif o.__class__==int:
            return "WhiteInteger"
        else:
            return "WhiteLeft"



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'xulpymoney/ui/frmAccountsReport.ui'
#
# Created by: PyQt5 UI code generator 5.12.2
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_frmAccountsReport(object):
    def setupUi(self, frmAccountsReport):
        frmAccountsReport.setObjectName("frmAccountsReport")
        frmAccountsReport.setWindowModality(QtCore.Qt.WindowModal)
        frmAccountsReport.resize(813, 607)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/xulpymoney/report2.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        frmAccountsReport.setWindowIcon(icon)
        frmAccountsReport.setModal(True)
        self._2 = QtWidgets.QVBoxLayout(frmAccountsReport)
        self._2.setObjectName("_2")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout()
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.lblTitulo = QtWidgets.QLabel(frmAccountsReport)
        font = QtGui.QFont()
        font.setPointSize(14)
        font.setBold(True)
        font.setWeight(75)
        self.lblTitulo.setFont(font)
        self.lblTitulo.setStyleSheet("color: rgb(0, 128, 0);")
        self.lblTitulo.setText("")
        self.lblTitulo.setAlignment(QtCore.Qt.AlignCenter)
        self.lblTitulo.setObjectName("lblTitulo")
        self.verticalLayout_2.addWidget(self.lblTitulo)
        self.tab = QtWidgets.QTabWidget(frmAccountsReport)
        self.tab.setObjectName("tab")
        self.tabMovimientos = QtWidgets.QWidget()
        self.tabMovimientos.setObjectName("tabMovimientos")
        self.horizontalLayout_5 = QtWidgets.QHBoxLayout(self.tabMovimientos)
        self.horizontalLayout_5.setObjectName("horizontalLayout_5")
        self.verticalLayout_6 = QtWidgets.QVBoxLayout()
        self.verticalLayout_6.setObjectName("verticalLayout_6")
        self.horizontalLayout_4 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_4.setObjectName("horizontalLayout_4")
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_4.addItem(spacerItem)
        self.groupBox = QtWidgets.QGroupBox(self.tabMovimientos)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Minimum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.groupBox.sizePolicy().hasHeightForWidth())
        self.groupBox.setSizePolicy(sizePolicy)
        self.groupBox.setObjectName("groupBox")
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout(self.groupBox)
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.verticalLayout = QtWidgets.QVBoxLayout()
        self.verticalLayout.setObjectName("verticalLayout")
        self.formLayout = QtWidgets.QFormLayout()
        self.formLayout.setFieldGrowthPolicy(QtWidgets.QFormLayout.ExpandingFieldsGrow)
        self.formLayout.setObjectName("formLayout")
        self.label = QtWidgets.QLabel(self.groupBox)
        self.label.setObjectName("label")
        self.formLayout.setWidget(0, QtWidgets.QFormLayout.LabelRole, self.label)
        self.cmbEB = QtWidgets.QComboBox(self.groupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.cmbEB.sizePolicy().hasHeightForWidth())
        self.cmbEB.setSizePolicy(sizePolicy)
        self.cmbEB.setSizeAdjustPolicy(QtWidgets.QComboBox.AdjustToContents)
        self.cmbEB.setObjectName("cmbEB")
        self.formLayout.setWidget(0, QtWidgets.QFormLayout.FieldRole, self.cmbEB)
        self.label_2 = QtWidgets.QLabel(self.groupBox)
        self.label_2.setObjectName("label_2")
        self.formLayout.setWidget(1, QtWidgets.QFormLayout.LabelRole, self.label_2)
        self.txtAccount = QtWidgets.QLineEdit(self.groupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.txtAccount.sizePolicy().hasHeightForWidth())
        self.txtAccount.setSizePolicy(sizePolicy)
        self.txtAccount.setMinimumSize(QtCore.QSize(300, 0))
        self.txtAccount.setObjectName("txtAccount")
        self.formLayout.setWidget(1, QtWidgets.QFormLayout.FieldRole, self.txtAccount)
        self.label_8 = QtWidgets.QLabel(self.groupBox)
        self.label_8.setObjectName("label_8")
        self.formLayout.setWidget(2, QtWidgets.QFormLayout.LabelRole, self.label_8)
        self.cmbCurrency = QtWidgets.QComboBox(self.groupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.cmbCurrency.sizePolicy().hasHeightForWidth())
        self.cmbCurrency.setSizePolicy(sizePolicy)
        self.cmbCurrency.setObjectName("cmbCurrency")
        self.formLayout.setWidget(2, QtWidgets.QFormLayout.FieldRole, self.cmbCurrency)
        self.label_3 = QtWidgets.QLabel(self.groupBox)
        self.label_3.setObjectName("label_3")
        self.formLayout.setWidget(3, QtWidgets.QFormLayout.LabelRole, self.label_3)
        self.txtNumero = QtWidgets.QLineEdit(self.groupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.txtNumero.sizePolicy().hasHeightForWidth())
        self.txtNumero.setSizePolicy(sizePolicy)
        self.txtNumero.setObjectName("txtNumero")
        self.formLayout.setWidget(3, QtWidgets.QFormLayout.FieldRole, self.txtNumero)
        self.chkActiva = QtWidgets.QCheckBox(self.groupBox)
        self.chkActiva.setObjectName("chkActiva")
        self.formLayout.setWidget(4, QtWidgets.QFormLayout.FieldRole, self.chkActiva)
        self.verticalLayout.addLayout(self.formLayout)
        self.horizontalLayout_12 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_12.setObjectName("horizontalLayout_12")
        spacerItem1 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_12.addItem(spacerItem1)
        self.cmdDatos = QtWidgets.QPushButton(self.groupBox)
        self.cmdDatos.setObjectName("cmdDatos")
        self.horizontalLayout_12.addWidget(self.cmdDatos)
        spacerItem2 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_12.addItem(spacerItem2)
        self.verticalLayout.addLayout(self.horizontalLayout_12)
        self.horizontalLayout_2.addLayout(self.verticalLayout)
        self.horizontalLayout_4.addWidget(self.groupBox)
        spacerItem3 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_4.addItem(spacerItem3)
        self.verticalLayout_6.addLayout(self.horizontalLayout_4)
        self.groupBox_2 = QtWidgets.QGroupBox(self.tabMovimientos)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.groupBox_2.sizePolicy().hasHeightForWidth())
        self.groupBox_2.setSizePolicy(sizePolicy)
        self.groupBox_2.setObjectName("groupBox_2")
        self.horizontalLayout_3 = QtWidgets.QHBoxLayout(self.groupBox_2)
        self.horizontalLayout_3.setObjectName("horizontalLayout_3")
        self.verticalLayout_3 = QtWidgets.QVBoxLayout()
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        spacerItem4 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem4)
        self.wdgYM = wdgYearMonth(self.groupBox_2)
        self.wdgYM.setObjectName("wdgYM")
        self.horizontalLayout.addWidget(self.wdgYM)
        spacerItem5 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem5)
        self.verticalLayout_3.addLayout(self.horizontalLayout)
        self.tblOperaciones = myQTableWidget(self.groupBox_2)
        self.tblOperaciones.setContextMenuPolicy(QtCore.Qt.CustomContextMenu)
        self.tblOperaciones.setAlternatingRowColors(True)
        self.tblOperaciones.setSelectionMode(QtWidgets.QAbstractItemView.SingleSelection)
        self.tblOperaciones.setSelectionBehavior(QtWidgets.QAbstractItemView.SelectRows)
        self.tblOperaciones.setObjectName("tblOperaciones")
        self.tblOperaciones.setColumnCount(6)
        self.tblOperaciones.setRowCount(0)
        item = QtWidgets.QTableWidgetItem()
        self.tblOperaciones.setHorizontalHeaderItem(0, item)
        item = QtWidgets.QTableWidgetItem()
        self.tblOperaciones.setHorizontalHeaderItem(1, item)
        item = QtWidgets.QTableWidgetItem()
        self.tblOperaciones.setHorizontalHeaderItem(2, item)
        item = QtWidgets.QTableWidgetItem()
        self.tblOperaciones.setHorizontalHeaderItem(3, item)
        item = QtWidgets.QTableWidgetItem()
        self.tblOperaciones.setHorizontalHeaderItem(4, item)
        item = QtWidgets.QTableWidgetItem()
        self.tblOperaciones.setHorizontalHeaderItem(5, item)
        self.tblOperaciones.horizontalHeader().setStretchLastSection(True)
        self.tblOperaciones.verticalHeader().setVisible(False)
        self.tblOperaciones.verticalHeader().setStretchLastSection(False)
        self.verticalLayout_3.addWidget(self.tblOperaciones)
        self.horizontalLayout_3.addLayout(self.verticalLayout_3)
        self.verticalLayout_6.addWidget(self.groupBox_2)
        self.horizontalLayout_5.addLayout(self.verticalLayout_6)
        icon1 = QtGui.QIcon()
        icon1.addPixmap(QtGui.QPixmap(":/xulpymoney/coins.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.tab.addTab(self.tabMovimientos, icon1, "")
        self.tabTarjetas = QtWidgets.QWidget()
        self.tabTarjetas.setObjectName("tabTarjetas")
        self.horizontalLayout_8 = QtWidgets.QHBoxLayout(self.tabTarjetas)
        self.horizontalLayout_8.setObjectName("horizontalLayout_8")
        self.verticalLayout_5 = QtWidgets.QVBoxLayout()
        self.verticalLayout_5.setObjectName("verticalLayout_5")
        self.groupBox_3 = QtWidgets.QGroupBox(self.tabTarjetas)
        self.groupBox_3.setObjectName("groupBox_3")
        self.horizontalLayout_9 = QtWidgets.QHBoxLayout(self.groupBox_3)
        self.horizontalLayout_9.setObjectName("horizontalLayout_9")
        self.verticalLayout_4 = QtWidgets.QVBoxLayout()
        self.verticalLayout_4.setObjectName("verticalLayout_4")
        self.chkCreditCards = QtWidgets.QCheckBox(self.groupBox_3)
        self.chkCreditCards.setObjectName("chkCreditCards")
        self.verticalLayout_4.addWidget(self.chkCreditCards)
        self.tblCreditCards = myQTableWidget(self.groupBox_3)
        self.tblCreditCards.setContextMenuPolicy(QtCore.Qt.CustomContextMenu)
        self.tblCreditCards.setAlternatingRowColors(True)
        self.tblCreditCards.setSelectionMode(QtWidgets.QAbstractItemView.SingleSelection)
        self.tblCreditCards.setSelectionBehavior(QtWidgets.QAbstractItemView.SelectRows)
        self.tblCreditCards.setObjectName("tblCreditCards")
        self.tblCreditCards.setColumnCount(6)
        self.tblCreditCards.setRowCount(0)
        item = QtWidgets.QTableWidgetItem()
        self.tblCreditCards.setHorizontalHeaderItem(0, item)
        item = QtWidgets.QTableWidgetItem()
        self.tblCreditCards.setHorizontalHeaderItem(1, item)
        item = QtWidgets.QTableWidgetItem()
        self.tblCreditCards.setHorizontalHeaderItem(2, item)
        item = QtWidgets.QTableWidgetItem()
        self.tblCreditCards.setHorizontalHeaderItem(3, item)
        item = QtWidgets.QTableWidgetItem()
        self.tblCreditCards.setHorizontalHeaderItem(4, item)
        item = QtWidgets.QTableWidgetItem()
        self.tblCreditCards.setHorizontalHeaderItem(5, item)
        self.tblCreditCards.verticalHeader().setVisible(False)
        self.verticalLayout_4.addWidget(self.tblCreditCards)
        self.horizontalLayout_9.addLayout(self.verticalLayout_4)
        self.verticalLayout_5.addWidget(self.groupBox_3)
        self.layDiferido = QtWidgets.QHBoxLayout()
        self.layDiferido.setObjectName("layDiferido")
        self.tabOpertarjetasDiferidas = QtWidgets.QTabWidget(self.tabTarjetas)
        self.tabOpertarjetasDiferidas.setEnabled(False)
        self.tabOpertarjetasDiferidas.setObjectName("tabOpertarjetasDiferidas")
        self.tab_2 = QtWidgets.QWidget()
        self.tab_2.setObjectName("tab_2")
        self.horizontalLayout_6 = QtWidgets.QHBoxLayout(self.tab_2)
        self.horizontalLayout_6.setObjectName("horizontalLayout_6")
        self.grpOperTarjetas = QtWidgets.QGroupBox(self.tab_2)
        self.grpOperTarjetas.setCheckable(False)
        self.grpOperTarjetas.setObjectName("grpOperTarjetas")
        self.verticalLayout_7 = QtWidgets.QVBoxLayout(self.grpOperTarjetas)
        self.verticalLayout_7.setObjectName("verticalLayout_7")
        self.tblCreditCardOpers = myQTableWidget(self.grpOperTarjetas)
        self.tblCreditCardOpers.setContextMenuPolicy(QtCore.Qt.CustomContextMenu)
        self.tblCreditCardOpers.setEditTriggers(QtWidgets.QAbstractItemView.NoEditTriggers)
        self.tblCreditCardOpers.setAlternatingRowColors(True)
        self.tblCreditCardOpers.setSelectionMode(QtWidgets.QAbstractItemView.MultiSelection)
        self.tblCreditCardOpers.setSelectionBehavior(QtWidgets.QAbstractItemView.SelectRows)
        self.tblCreditCardOpers.setObjectName("tblCreditCardOpers")
        self.tblCreditCardOpers.setColumnCount(4)
        self.tblCreditCardOpers.setRowCount(0)
        item = QtWidgets.QTableWidgetItem()
        self.tblCreditCardOpers.setHorizontalHeaderItem(0, item)
        item = QtWidgets.QTableWidgetItem()
        self.tblCreditCardOpers.setHorizontalHeaderItem(1, item)
        item = QtWidgets.QTableWidgetItem()
        self.tblCreditCardOpers.setHorizontalHeaderItem(2, item)
        item = QtWidgets.QTableWidgetItem()
        self.tblCreditCardOpers.setHorizontalHeaderItem(3, item)
        self.tblCreditCardOpers.verticalHeader().setVisible(False)
        self.verticalLayout_7.addWidget(self.tblCreditCardOpers)
        self.horizontalLayout_6.addWidget(self.grpOperTarjetas)
        self.verticalLayout_10 = QtWidgets.QVBoxLayout()
        self.verticalLayout_10.setObjectName("verticalLayout_10")
        spacerItem6 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout_10.addItem(spacerItem6)
        self.grpPago = QtWidgets.QGroupBox(self.tab_2)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.grpPago.sizePolicy().hasHeightForWidth())
        self.grpPago.setSizePolicy(sizePolicy)
        self.grpPago.setMaximumSize(QtCore.QSize(400, 16777215))
        self.grpPago.setObjectName("grpPago")
        self.horizontalLayout_11 = QtWidgets.QHBoxLayout(self.grpPago)
        self.horizontalLayout_11.setObjectName("horizontalLayout_11")
        self.verticalLayout_8 = QtWidgets.QVBoxLayout()
        self.verticalLayout_8.setObjectName("verticalLayout_8")
        self.lblPago = QtWidgets.QLabel(self.grpPago)
        font = QtGui.QFont()
        font.setPointSize(12)
        font.setBold(True)
        font.setWeight(75)
        self.lblPago.setFont(font)
        self.lblPago.setAlignment(QtCore.Qt.AlignCenter)
        self.lblPago.setObjectName("lblPago")
        self.verticalLayout_8.addWidget(self.lblPago)
        self.horizontalLayout_10 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_10.setObjectName("horizontalLayout_10")
        self.wdgDtPago = wdgDatetime(self.grpPago)
        self.wdgDtPago.setObjectName("wdgDtPago")
        self.horizontalLayout_10.addWidget(self.wdgDtPago)
        self.verticalLayout_8.addLayout(self.horizontalLayout_10)
        self.cmdPago = QtWidgets.QPushButton(self.grpPago)
        self.cmdPago.setObjectName("cmdPago")
        self.verticalLayout_8.addWidget(self.cmdPago)
        self.horizontalLayout_11.addLayout(self.verticalLayout_8)
        self.verticalLayout_10.addWidget(self.grpPago)
        spacerItem7 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout_10.addItem(spacerItem7)
        self.horizontalLayout_6.addLayout(self.verticalLayout_10)
        self.tabOpertarjetasDiferidas.addTab(self.tab_2, "")
        self.tab_3 = QtWidgets.QWidget()
        self.tab_3.setObjectName("tab_3")
        self.horizontalLayout_14 = QtWidgets.QHBoxLayout(self.tab_3)
        self.horizontalLayout_14.setObjectName("horizontalLayout_14")
        self.verticalLayout_9 = QtWidgets.QVBoxLayout()
        self.verticalLayout_9.setObjectName("verticalLayout_9")
        self.horizontalLayout_13 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_13.setObjectName("horizontalLayout_13")
        self.label_6 = QtWidgets.QLabel(self.tab_3)
        self.label_6.setObjectName("label_6")
        self.horizontalLayout_13.addWidget(self.label_6)
        self.cmbFechasPago = QtWidgets.QComboBox(self.tab_3)
        self.cmbFechasPago.setSizeAdjustPolicy(QtWidgets.QComboBox.AdjustToContents)
        self.cmbFechasPago.setObjectName("cmbFechasPago")
        self.horizontalLayout_13.addWidget(self.cmbFechasPago)
        spacerItem8 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_13.addItem(spacerItem8)
        self.label_7 = QtWidgets.QLabel(self.tab_3)
        self.label_7.setObjectName("label_7")
        self.horizontalLayout_13.addWidget(self.label_7)
        self.cmdDevolverPago = QtWidgets.QPushButton(self.tab_3)
        icon2 = QtGui.QIcon()
        icon2.addPixmap(QtGui.QPixmap(":/xulpmoney/tools-wizard.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.cmdDevolverPago.setIcon(icon2)
        self.cmdDevolverPago.setObjectName("cmdDevolverPago")
        self.horizontalLayout_13.addWidget(self.cmdDevolverPago)
        self.verticalLayout_9.addLayout(self.horizontalLayout_13)
        self.tblOpertarjetasHistoricas = myQTableWidget(self.tab_3)
        self.tblOpertarjetasHistoricas.setAlternatingRowColors(True)
        self.tblOpertarjetasHistoricas.setSelectionMode(QtWidgets.QAbstractItemView.SingleSelection)
        self.tblOpertarjetasHistoricas.setSelectionBehavior(QtWidgets.QAbstractItemView.SelectRows)
        self.tblOpertarjetasHistoricas.setObjectName("tblOpertarjetasHistoricas")
        self.tblOpertarjetasHistoricas.setColumnCount(0)
        self.tblOpertarjetasHistoricas.setRowCount(0)
        self.tblOpertarjetasHistoricas.verticalHeader().setVisible(False)
        self.verticalLayout_9.addWidget(self.tblOpertarjetasHistoricas)
        self.horizontalLayout_14.addLayout(self.verticalLayout_9)
        self.tabOpertarjetasDiferidas.addTab(self.tab_3, "")
        self.layDiferido.addWidget(self.tabOpertarjetasDiferidas)
        self.verticalLayout_5.addLayout(self.layDiferido)
        self.horizontalLayout_8.addLayout(self.verticalLayout_5)
        icon3 = QtGui.QIcon()
        icon3.addPixmap(QtGui.QPixmap(":/xulpymoney/visa2.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.tab.addTab(self.tabTarjetas, icon3, "")
        self.verticalLayout_2.addWidget(self.tab)
        self._2.addLayout(self.verticalLayout_2)
        self.actionOperationAdd = QtWidgets.QAction(frmAccountsReport)
        icon4 = QtGui.QIcon()
        icon4.addPixmap(QtGui.QPixmap(":/xulpymoney/add.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.actionOperationAdd.setIcon(icon4)
        self.actionOperationAdd.setObjectName("actionOperationAdd")
        self.actionOperationDelete = QtWidgets.QAction(frmAccountsReport)
        icon5 = QtGui.QIcon()
        icon5.addPixmap(QtGui.QPixmap(":/xulpymoney/list-remove.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.actionOperationDelete.setIcon(icon5)
        self.actionOperationDelete.setObjectName("actionOperationDelete")
        self.actionCreditCardAdd = QtWidgets.QAction(frmAccountsReport)
        self.actionCreditCardAdd.setIcon(icon4)
        self.actionCreditCardAdd.setObjectName("actionCreditCardAdd")
        self.actionCreditCardDelete = QtWidgets.QAction(frmAccountsReport)
        self.actionCreditCardDelete.setIcon(icon5)
        self.actionCreditCardDelete.setObjectName("actionCreditCardDelete")
        self.actionCreditCardOperAdd = QtWidgets.QAction(frmAccountsReport)
        self.actionCreditCardOperAdd.setIcon(icon4)
        self.actionCreditCardOperAdd.setObjectName("actionCreditCardOperAdd")
        self.actionCreditCardOperDelete = QtWidgets.QAction(frmAccountsReport)
        self.actionCreditCardOperDelete.setIcon(icon5)
        self.actionCreditCardOperDelete.setObjectName("actionCreditCardOperDelete")
        self.actionCreditCardEdit = QtWidgets.QAction(frmAccountsReport)
        icon6 = QtGui.QIcon()
        icon6.addPixmap(QtGui.QPixmap(":/xulpymoney/editar.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.actionCreditCardEdit.setIcon(icon6)
        self.actionCreditCardEdit.setObjectName("actionCreditCardEdit")
        self.actionCreditCardActivate = QtWidgets.QAction(frmAccountsReport)
        self.actionCreditCardActivate.setCheckable(True)
        self.actionCreditCardActivate.setObjectName("actionCreditCardActivate")
        self.actionOperationEdit = QtWidgets.QAction(frmAccountsReport)
        self.actionOperationEdit.setIcon(icon6)
        self.actionOperationEdit.setObjectName("actionOperationEdit")
        self.actionCreditCardOperEdit = QtWidgets.QAction(frmAccountsReport)
        self.actionCreditCardOperEdit.setIcon(icon6)
        self.actionCreditCardOperEdit.setObjectName("actionCreditCardOperEdit")
        self.actionTransferDelete = QtWidgets.QAction(frmAccountsReport)
        self.actionTransferDelete.setIcon(icon5)
        self.actionTransferDelete.setObjectName("actionTransferDelete")
        self.actionInvestmentOperationEdit = QtWidgets.QAction(frmAccountsReport)
        icon7 = QtGui.QIcon()
        icon7.addPixmap(QtGui.QPixmap(":/xulpymoney/document-edit.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.actionInvestmentOperationEdit.setIcon(icon7)
        self.actionInvestmentOperationEdit.setObjectName("actionInvestmentOperationEdit")
        self.actionInvestmentOperationDelete = QtWidgets.QAction(frmAccountsReport)
        self.actionInvestmentOperationDelete.setIcon(icon5)
        self.actionInvestmentOperationDelete.setObjectName("actionInvestmentOperationDelete")
        self.actionCreditCardOperRefund = QtWidgets.QAction(frmAccountsReport)
        icon8 = QtGui.QIcon()
        icon8.addPixmap(QtGui.QPixmap(":/xulpymoney/Money.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.actionCreditCardOperRefund.setIcon(icon8)
        self.actionCreditCardOperRefund.setObjectName("actionCreditCardOperRefund")
        self.actionConceptReport = QtWidgets.QAction(frmAccountsReport)
        icon9 = QtGui.QIcon()
        icon9.addPixmap(QtGui.QPixmap(":/xulpymoney/hucha.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.actionConceptReport.setIcon(icon9)
        self.actionConceptReport.setObjectName("actionConceptReport")

        self.retranslateUi(frmAccountsReport)
        self.tab.setCurrentIndex(0)
        self.tabOpertarjetasDiferidas.setCurrentIndex(0)
        QtCore.QMetaObject.connectSlotsByName(frmAccountsReport)

    def retranslateUi(self, frmAccountsReport):
        _translate = QtCore.QCoreApplication.translate
        frmAccountsReport.setWindowTitle(_translate("frmAccountsReport", "Account report"))
        self.groupBox.setTitle(_translate("frmAccountsReport", "Account data"))
        self.label.setText(_translate("frmAccountsReport", "Select the bank it belongs to"))
        self.label_2.setText(_translate("frmAccountsReport", "Account name"))
        self.label_8.setText(_translate("frmAccountsReport", "Account currency"))
        self.label_3.setText(_translate("frmAccountsReport", "Account number"))
        self.chkActiva.setText(_translate("frmAccountsReport", "Is active?"))
        self.cmdDatos.setText(_translate("frmAccountsReport", "Update"))
        self.groupBox_2.setTitle(_translate("frmAccountsReport", "Account movements"))
        item = self.tblOperaciones.horizontalHeaderItem(0)
        item.setText(_translate("frmAccountsReport", "Date"))
        item = self.tblOperaciones.horizontalHeaderItem(1)
        item.setText(_translate("frmAccountsReport", "Concept"))
        item = self.tblOperaciones.horizontalHeaderItem(2)
        item.setText(_translate("frmAccountsReport", "Amount"))
        item = self.tblOperaciones.horizontalHeaderItem(3)
        item.setText(_translate("frmAccountsReport", "Balance"))
        item = self.tblOperaciones.horizontalHeaderItem(4)
        item.setText(_translate("frmAccountsReport", "Comment"))
        item = self.tblOperaciones.horizontalHeaderItem(5)
        item.setText(_translate("frmAccountsReport", "Id"))
        self.tab.setTabText(self.tab.indexOf(self.tabMovimientos), _translate("frmAccountsReport", "Account"))
        self.groupBox_3.setTitle(_translate("frmAccountsReport", "Credit card list"))
        self.chkCreditCards.setText(_translate("frmAccountsReport", "Show inactive credit cards"))
        item = self.tblCreditCards.horizontalHeaderItem(0)
        item.setText(_translate("frmAccountsReport", "Credit card"))
        item = self.tblCreditCards.horizontalHeaderItem(1)
        item.setText(_translate("frmAccountsReport", "Number"))
        item = self.tblCreditCards.horizontalHeaderItem(2)
        item.setText(_translate("frmAccountsReport", "Active"))
        item = self.tblCreditCards.horizontalHeaderItem(3)
        item.setText(_translate("frmAccountsReport", "Delayed payment"))
        item = self.tblCreditCards.horizontalHeaderItem(4)
        item.setText(_translate("frmAccountsReport", "Maximum balance"))
        item = self.tblCreditCards.horizontalHeaderItem(5)
        item.setText(_translate("frmAccountsReport", "Balance"))
        self.grpOperTarjetas.setTitle(_translate("frmAccountsReport", "Selected Credit Card delayed operations"))
        item = self.tblCreditCardOpers.horizontalHeaderItem(0)
        item.setText(_translate("frmAccountsReport", "Date"))
        item = self.tblCreditCardOpers.horizontalHeaderItem(1)
        item.setText(_translate("frmAccountsReport", "Concept"))
        item = self.tblCreditCardOpers.horizontalHeaderItem(2)
        item.setText(_translate("frmAccountsReport", "Amount"))
        item = self.tblCreditCardOpers.horizontalHeaderItem(3)
        item.setText(_translate("frmAccountsReport", "Comment"))
        self.grpPago.setTitle(_translate("frmAccountsReport", "Selected operations delayed payment"))
        self.lblPago.setText(_translate("frmAccountsReport", "XXXX €"))
        self.cmdPago.setText(_translate("frmAccountsReport", "Pay"))
        self.tabOpertarjetasDiferidas.setTabText(self.tabOpertarjetasDiferidas.indexOf(self.tab_2), _translate("frmAccountsReport", "Selected Credit Card delayed operations"))
        self.label_6.setText(_translate("frmAccountsReport", "Select a payment"))
        self.label_7.setText(_translate("frmAccountsReport", "Pay back"))
        self.cmdDevolverPago.setText(_translate("frmAccountsReport", "Pay back"))
        self.tabOpertarjetasDiferidas.setTabText(self.tabOpertarjetasDiferidas.indexOf(self.tab_3), _translate("frmAccountsReport", "Deferred payments for the selected card report"))
        self.tab.setTabText(self.tab.indexOf(self.tabTarjetas), _translate("frmAccountsReport", "Associated credit cards"))
        self.actionOperationAdd.setText(_translate("frmAccountsReport", "New operation"))
        self.actionOperationAdd.setToolTip(_translate("frmAccountsReport", "New operation"))
        self.actionOperationDelete.setText(_translate("frmAccountsReport", "Delete operation"))
        self.actionOperationDelete.setToolTip(_translate("frmAccountsReport", "Delete operation"))
        self.actionCreditCardAdd.setText(_translate("frmAccountsReport", "New credit card"))
        self.actionCreditCardDelete.setText(_translate("frmAccountsReport", "Delete credit card"))
        self.actionCreditCardDelete.setToolTip(_translate("frmAccountsReport", "Delete credit card"))
        self.actionCreditCardOperAdd.setText(_translate("frmAccountsReport", "New credit card operation"))
        self.actionCreditCardOperAdd.setToolTip(_translate("frmAccountsReport", "New credit card operation"))
        self.actionCreditCardOperDelete.setText(_translate("frmAccountsReport", "Delete credit card operation"))
        self.actionCreditCardOperDelete.setToolTip(_translate("frmAccountsReport", "Delete credit card operation"))
        self.actionCreditCardEdit.setText(_translate("frmAccountsReport", "Update credit card"))
        self.actionCreditCardEdit.setToolTip(_translate("frmAccountsReport", "Update credit card"))
        self.actionCreditCardActivate.setText(_translate("frmAccountsReport", "Is active?"))
        self.actionOperationEdit.setText(_translate("frmAccountsReport", "Update operation"))
        self.actionOperationEdit.setToolTip(_translate("frmAccountsReport", "Update operation"))
        self.actionCreditCardOperEdit.setText(_translate("frmAccountsReport", "Update credit card operation"))
        self.actionCreditCardOperEdit.setToolTip(_translate("frmAccountsReport", "Update credit card operation"))
        self.actionTransferDelete.setText(_translate("frmAccountsReport", "Delete transfer"))
        self.actionTransferDelete.setToolTip(_translate("frmAccountsReport", "Delete transfer"))
        self.actionInvestmentOperationEdit.setText(_translate("frmAccountsReport", "Edit associated investment operation"))
        self.actionInvestmentOperationEdit.setToolTip(_translate("frmAccountsReport", "Edit associated investment operation"))
        self.actionInvestmentOperationDelete.setText(_translate("frmAccountsReport", "Delete associated investment operation"))
        self.actionInvestmentOperationDelete.setToolTip(_translate("frmAccountsReport", "Delete associated investment operation"))
        self.actionCreditCardOperRefund.setText(_translate("frmAccountsReport", "Credit card operation refund"))
        self.actionCreditCardOperRefund.setToolTip(_translate("frmAccountsReport", "Credit card operation refund"))
        self.actionConceptReport.setText(_translate("frmAccountsReport", "Show concept historical report"))
        self.actionConceptReport.setToolTip(_translate("frmAccountsReport", "Show concept historical report"))


from xulpymoney.ui.myqtablewidget import myQTableWidget
from xulpymoney.ui.wdgDatetime import wdgDatetime
from xulpymoney.ui.wdgYearMonth import wdgYearMonth
import xulpymoney.images.xulpymoney_rc



import datetime
from PyQt5.QtCore import Qt, pyqtSlot
from PyQt5.QtWidgets import QDialog
from xulpymoney.ui.Ui_frmSellingPoint import Ui_frmSellingPoint
from xulpymoney.libxulpymoney import Money,  Quote, InvestmentOperationCurrentHomogeneusManager,  Percentage
from xulpymoney.libxulpymoneyfunctions import qmessagebox
from xulpymoney.libxulpymoneytypes import eMoneyCurrency
from decimal import Decimal

class frmSellingPoint(QDialog, Ui_frmSellingPoint):
    def __init__(self, mem,  inversion ,   parent=None):
        QDialog.__init__(self, parent)
        self.setupUi(self)
        self.showMaximized()
        self.mem=mem
        self.investment=inversion
        
        if self.investment.id==None:
            qmessagebox(self.tr("You can't set a selling price to a unsaved investment"))
            return

        if self.investment.op_actual.length()==0:
            qmessagebox(self.tr("You don't have shares to sale in this investment"))
            return
        
        self.puntoventa=Decimal(0)#Guarda el resultado de los cálculos
        self.operinversiones=None 

        if self.mem.gainsyear==True:
            self.chkGainsTime.setCheckState(Qt.Checked)
        else:
            self.chkGainsTime.setEnabled(False)

        self.table.settings(self.mem, "frmSellingPoint")
        self.tableSP.settings(self.mem, "frmSellingPoint")
        
        self.spnGainsPercentage.setValue(float(self.mem.settingsdb.value("frmSellingPoint/lastgainpercentage",  10)))
        
        self.lblPriceSymbol.setText(self.investment.product.currency.symbol)
        self.lblAmountSymbol.setText(self.investment.account.currency.symbol)
        
    def __calcular(self):
        #Setting self.operinversiones variable
        if self.chkPonderanAll.checkState()==Qt.Checked:#Results are in self.mem.localcurrency
            self.operinversiones=self.mem.data.investments_active().investment_merging_current_operations_with_same_product(self.investment.product).op_actual
            self.operinversiones.myqtablewidget(self.table)
        else:#Results in account currency
            self.operinversiones=InvestmentOperationCurrentHomogeneusManager(self.mem, self.investment)
            if self.chkGainsTime.checkState()==Qt.Checked:
                self.operinversiones=self.investment.op_actual.ObjectManager_copy_until_datetime(self.mem.localzone.now()-datetime.timedelta(days=365), self.mem, self.investment)
            else:
                self.operinversiones=self.investment.op_actual.ObjectManager_copy_until_datetime(None, self.mem, self.investment)
            self.operinversiones.myqtablewidget(self.table, self.investment.product.result.basic.last,  eMoneyCurrency.Account)
        sumacciones=self.operinversiones.shares()
        
        #Calculations
        if sumacciones==Decimal(0):
            self.puntoventa=Money(self.mem, 0, self.investment.account.currency)
        else:
            if self.radTPC.isChecked()==True:
                percentage=Percentage(self.spnGainsPercentage.value(), 100)
                self.puntoventa=self.operinversiones.selling_price_to_gain_percentage_of_invested(percentage, eMoneyCurrency.Account)
            elif self.radPrice.isChecked()==True:
                if self.txtPrice.isValid():#Si hay un numero bien
                    self.puntoventa=Money(self.mem,  self.txtPrice.decimal(),  self.investment.product.currency)
                    self.cmd.setEnabled(True)
                else:
                    self.puntoventa=Money(self.mem, 0, self.investment.product.currency)
                    self.cmd.setEnabled(False)
            elif self.radGain.isChecked()==True:
                if self.txtGanancia.isValid():#Si hay un numero bien
                    self.puntoventa=self.operinversiones.selling_price_to_gain_money(Money(self.mem, self.txtGanancia.decimal(), self.investment.product.currency))
                    self.cmd.setEnabled(True)
                else:
                    self.puntoventa=Money(self.mem, 0, self.investment.account.currency)
                    self.cmd.setEnabled(False)

        quote=Quote(self.mem).init__create(self.investment.product, self.mem.localzone.now(), self.puntoventa.amount)
        self.tab.setTabText(1, self.tr("Selling point: {0}".format(self.puntoventa)))
        self.tab.setTabText(0, self.tr("Current state: {0}".format(quote.money())))
        self.operinversiones.myqtablewidget(self.tableSP, quote, eMoneyCurrency.Account) 
        
        if self.chkPonderanAll.checkState()==Qt.Checked:
            self.cmd.setText(self.tr("Set selling price to all investments  of {0} to gain {1}").format(self.puntoventa, self.operinversiones.pendiente(quote, eMoneyCurrency.Account)))
        else:
            self.cmd.setText(self.tr("Set {0} shares selling price to {1} to gain {2}").format(sumacciones, self.puntoventa, self.operinversiones.pendiente(quote, eMoneyCurrency.Account)))

    def on_radTPC_clicked(self):
        self.__calcular()
        
    def on_radPrice_clicked(self):
        self.__calcular()
        
    def on_radGain_clicked(self):
        self.__calcular()
        
    @pyqtSlot(float) 
    def on_spnGainsPercentage_valueChanged(self, value):
        self.__calcular()
        
    def on_txtGanancia_textChanged(self):
        self.__calcular()
        
    def on_txtPrice_textChanged(self):
        self.__calcular()
        
    def on_chkPonderanAll_stateChanged(self, state):
        self.__calcular()
        
    def on_chkGainsTime_stateChanged(self, state):
        self.__calcular()

    @pyqtSlot() 
    def on_cmd_released(self):
        if self.chkPonderanAll.checkState()==Qt.Checked:
            for inv in self.mem.data.investments_active().arr:
                if inv.product.id==self.investment.product.id:
                    inv.venta=self.puntoventa.amount
                    inv.save()
            self.mem.con.commit()
        
        #Save in settings the last selling percentage, if that's the case
        if self.radTPC.isChecked():
            percentage=Decimal(self.spnGainsPercentage.value())
            self.mem.settingsdb.setValue("frmSellingPoint/lastgainpercentage", percentage)
        self.done(0)



## @brief Package to manage postgresql admin functionss
## THIS IS FROM XULPYMONEY PACKAGE IF YOU NEED THIS MODULE PLEASE SYNC IT FROM THERE, FOR EXAMPLE
## @code
##       print ("Copying admin_pg.py from Xulpymoney project")
##        os.chdir("your directory)
##        os.remove("admin_pg.py")
##        os.system("wget https://raw.githubusercontent.com/Turulomio/xulpymoney/master/xulpymoney/admin_pg.py  --no-clobber")
##        os.system("sed -i -e '3i ## THIS FILE HAS BEEN DOWNLOADED AT {} FROM https://github.com/Turulomio/xulpymoney/xulpymoney/admin_pg.py.' admin_pg.py".format(datetime.datetime.now()))
## @encode

import io
import logging
from .connection_pg import Connection

class AdminPG:
    def __init__(self, user, password, server, port):
        """connection is an object Connection to a database"""
        self.con=Connection().init__create(user, password, server, port, "postgres")
        self.con.connect()
        self.con.setAutocommit(True)

    def create_db(self, database):
        """It has database parameter, due to I connect to template to create database"""
        if self.con.is_superuser():
            cur=self.con.cursor()
            cur.execute("create database {0};".format(database))
        else:
            logging.critical ("You need to be superuser to create database")
            return False
        
        
    def db_exists(self, database):
        """Hace conexi´on automatica a template usando la con """
        cur=self.con.cursor()
        cur.execute("SELECT 1 AS result FROM pg_database WHERE datname=%s", (database, ))
        
        if cur.rowcount==1:
            cur.close()
            return True
        cur.close()
        return False

    def drop_db(self, database):
        """It has database parameter, due to I connect to template to drop database"""
        
        if self.db_exists(database)==False:
            logging.info("Database doesn't exist")
            return True
        
        if self.con.is_superuser():
            try:
                cur=self.cursor()
                cur.execute("drop database {0};".format(database))
            except:
                logging.error ("Error in drop()")
            finally:
                cur.close()
                return False
            logging.info("Database droped")
            return True
        else:
            logging.warning ("You need to be superuser to drop a database")
            return False
        

        
    ## Returns a Connection object to a database using Admin connection information
    def connect_to_database(self, database):
        con=Connection().init__create(self.con.user, self.con.password, self.con.server, self.con.port, database)
        con.connect()
        return con
        
    def copy(self, con_origin, sql,  table_destiny ):
        """Used to copy between tables, and sql to table_destiny, table origin and destiny must have the same structure"""
        if sql.__class__==bytes:
            sql=sql.decode('UTF-8')
        f=io.StringIO()
        cur_origin=con_origin.cursor()
        cur_origin.copy_expert("copy ({}) to stdout".format(sql), f)
        cur_origin.close()
        f.seek(0)
        cur_destiny=self.con.cursor()
        cur_destiny.copy_from(f, table_destiny)
        cur_destiny.close()
        f.seek(0)
        logging.debug (f.read())
        f.close()



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'xulpymoney/ui/wdgSimulations.ui'
#
# Created by: PyQt5 UI code generator 5.12.2
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_wdgSimulations(object):
    def setupUi(self, wdgSimulations):
        wdgSimulations.setObjectName("wdgSimulations")
        wdgSimulations.resize(1048, 499)
        self.horizontalLayout = QtWidgets.QHBoxLayout(wdgSimulations)
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.verticalLayout = QtWidgets.QVBoxLayout()
        self.verticalLayout.setObjectName("verticalLayout")
        self.lblTitulo = QtWidgets.QLabel(wdgSimulations)
        font = QtGui.QFont()
        font.setPointSize(14)
        font.setBold(True)
        font.setWeight(75)
        self.lblTitulo.setFont(font)
        self.lblTitulo.setAlignment(QtCore.Qt.AlignCenter)
        self.lblTitulo.setObjectName("lblTitulo")
        self.verticalLayout.addWidget(self.lblTitulo)
        self.horizontalLayout_4 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_4.setObjectName("horizontalLayout_4")
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_4.addItem(spacerItem)
        self.lblPixmap = QtWidgets.QLabel(wdgSimulations)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.lblPixmap.sizePolicy().hasHeightForWidth())
        self.lblPixmap.setSizePolicy(sizePolicy)
        self.lblPixmap.setMinimumSize(QtCore.QSize(48, 48))
        self.lblPixmap.setMaximumSize(QtCore.QSize(48, 48))
        self.lblPixmap.setPixmap(QtGui.QPixmap(":/xulpymoney/replication.png"))
        self.lblPixmap.setScaledContents(True)
        self.lblPixmap.setAlignment(QtCore.Qt.AlignCenter)
        self.lblPixmap.setObjectName("lblPixmap")
        self.horizontalLayout_4.addWidget(self.lblPixmap)
        spacerItem1 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_4.addItem(spacerItem1)
        self.verticalLayout.addLayout(self.horizontalLayout_4)
        self.groupBox = QtWidgets.QGroupBox(wdgSimulations)
        self.groupBox.setObjectName("groupBox")
        self.horizontalLayout_3 = QtWidgets.QHBoxLayout(self.groupBox)
        self.horizontalLayout_3.setObjectName("horizontalLayout_3")
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.tblSimulations = myQTableWidget(self.groupBox)
        self.tblSimulations.setSelectionMode(QtWidgets.QAbstractItemView.SingleSelection)
        self.tblSimulations.setSelectionBehavior(QtWidgets.QAbstractItemView.SelectRows)
        self.tblSimulations.setObjectName("tblSimulations")
        self.tblSimulations.setColumnCount(0)
        self.tblSimulations.setRowCount(0)
        self.horizontalLayout_2.addWidget(self.tblSimulations)
        self.verticalLayout_2 = QtWidgets.QVBoxLayout()
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.cmdCreate = QtWidgets.QPushButton(self.groupBox)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/xulpymoney/add.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.cmdCreate.setIcon(icon)
        self.cmdCreate.setObjectName("cmdCreate")
        self.verticalLayout_2.addWidget(self.cmdCreate)
        self.cmdDelete = QtWidgets.QPushButton(self.groupBox)
        self.cmdDelete.setEnabled(False)
        icon1 = QtGui.QIcon()
        icon1.addPixmap(QtGui.QPixmap(":/xulpymoney/list-remove.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.cmdDelete.setIcon(icon1)
        self.cmdDelete.setObjectName("cmdDelete")
        self.verticalLayout_2.addWidget(self.cmdDelete)
        self.cmdConnect = QtWidgets.QPushButton(self.groupBox)
        self.cmdConnect.setEnabled(False)
        icon2 = QtGui.QIcon()
        icon2.addPixmap(QtGui.QPixmap(":/xulpymoney/transfer.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.cmdConnect.setIcon(icon2)
        self.cmdConnect.setObjectName("cmdConnect")
        self.verticalLayout_2.addWidget(self.cmdConnect)
        self.horizontalLayout_2.addLayout(self.verticalLayout_2)
        self.horizontalLayout_3.addLayout(self.horizontalLayout_2)
        self.verticalLayout.addWidget(self.groupBox)
        self.horizontalLayout.addLayout(self.verticalLayout)

        self.retranslateUi(wdgSimulations)
        QtCore.QMetaObject.connectSlotsByName(wdgSimulations)

    def retranslateUi(self, wdgSimulations):
        _translate = QtCore.QCoreApplication.translate
        wdgSimulations.setWindowTitle(_translate("wdgSimulations", "Form"))
        self.lblTitulo.setText(_translate("wdgSimulations", "Xulpymoney simulations"))
        self.groupBox.setTitle(_translate("wdgSimulations", "Simulations"))
        self.cmdCreate.setText(_translate("wdgSimulations", "Create"))
        self.cmdDelete.setText(_translate("wdgSimulations", "Delete"))
        self.cmdConnect.setText(_translate("wdgSimulations", "Connect"))


from xulpymoney.ui.myqtablewidget import myQTableWidget
import xulpymoney.images.xulpymoney_rc



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'xulpymoney/ui/wdgCuriosity.ui'
#
# Created by: PyQt5 UI code generator 5.12.2
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_wdgCuriosity(object):
    def setupUi(self, wdgCuriosity):
        wdgCuriosity.setObjectName("wdgCuriosity")
        wdgCuriosity.resize(508, 78)
        self.horizontalLayout_4 = QtWidgets.QHBoxLayout(wdgCuriosity)
        self.horizontalLayout_4.setObjectName("horizontalLayout_4")
        self.horizontalLayout_3 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_3.setObjectName("horizontalLayout_3")
        self.lblPixmap = QtWidgets.QLabel(wdgCuriosity)
        self.lblPixmap.setMinimumSize(QtCore.QSize(32, 32))
        self.lblPixmap.setMaximumSize(QtCore.QSize(32, 32))
        self.lblPixmap.setText("")
        self.lblPixmap.setPixmap(QtGui.QPixmap(":/xulpymoney/curiosity.png"))
        self.lblPixmap.setScaledContents(True)
        self.lblPixmap.setObjectName("lblPixmap")
        self.horizontalLayout_3.addWidget(self.lblPixmap)
        self.verticalLayout = QtWidgets.QVBoxLayout()
        self.verticalLayout.setObjectName("verticalLayout")
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.label_2 = QtWidgets.QLabel(wdgCuriosity)
        self.label_2.setMinimumSize(QtCore.QSize(10, 0))
        self.label_2.setMaximumSize(QtCore.QSize(10, 16777215))
        self.label_2.setObjectName("label_2")
        self.horizontalLayout.addWidget(self.label_2)
        self.lblTitle = QtWidgets.QLabel(wdgCuriosity)
        font = QtGui.QFont()
        font.setPointSize(12)
        font.setBold(True)
        font.setItalic(False)
        font.setWeight(75)
        self.lblTitle.setFont(font)
        self.lblTitle.setAlignment(QtCore.Qt.AlignJustify|QtCore.Qt.AlignVCenter)
        self.lblTitle.setWordWrap(True)
        self.lblTitle.setObjectName("lblTitle")
        self.horizontalLayout.addWidget(self.lblTitle)
        self.verticalLayout.addLayout(self.horizontalLayout)
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.label = QtWidgets.QLabel(wdgCuriosity)
        self.label.setMinimumSize(QtCore.QSize(60, 0))
        self.label.setMaximumSize(QtCore.QSize(60, 16777215))
        self.label.setText("")
        self.label.setObjectName("label")
        self.horizontalLayout_2.addWidget(self.label)
        self.lbl = QtWidgets.QLabel(wdgCuriosity)
        font = QtGui.QFont()
        font.setPointSize(11)
        self.lbl.setFont(font)
        self.lbl.setAlignment(QtCore.Qt.AlignJustify|QtCore.Qt.AlignVCenter)
        self.lbl.setWordWrap(True)
        self.lbl.setObjectName("lbl")
        self.horizontalLayout_2.addWidget(self.lbl)
        self.verticalLayout.addLayout(self.horizontalLayout_2)
        self.horizontalLayout_3.addLayout(self.verticalLayout)
        self.horizontalLayout_4.addLayout(self.horizontalLayout_3)

        self.retranslateUi(wdgCuriosity)
        QtCore.QMetaObject.connectSlotsByName(wdgCuriosity)

    def retranslateUi(self, wdgCuriosity):
        pass


import xulpymoney.images.xulpymoney_rc



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'xulpymoney/ui/frmInvestmentReport.ui'
#
# Created by: PyQt5 UI code generator 5.12.2
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_frmInvestmentReport(object):
    def setupUi(self, frmInvestmentReport):
        frmInvestmentReport.setObjectName("frmInvestmentReport")
        frmInvestmentReport.setWindowModality(QtCore.Qt.WindowModal)
        frmInvestmentReport.resize(860, 539)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/xulpymoney/report2.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        frmInvestmentReport.setWindowIcon(icon)
        frmInvestmentReport.setModal(True)
        self.verticalLayout_10 = QtWidgets.QVBoxLayout(frmInvestmentReport)
        self.verticalLayout_10.setObjectName("verticalLayout_10")
        self.verticalLayout_9 = QtWidgets.QVBoxLayout()
        self.verticalLayout_9.setObjectName("verticalLayout_9")
        self.lblTitulo = QtWidgets.QLabel(frmInvestmentReport)
        font = QtGui.QFont()
        font.setPointSize(14)
        font.setBold(True)
        font.setWeight(75)
        self.lblTitulo.setFont(font)
        self.lblTitulo.setStyleSheet("color: rgb(0, 128, 0);")
        self.lblTitulo.setAlignment(QtCore.Qt.AlignCenter)
        self.lblTitulo.setObjectName("lblTitulo")
        self.verticalLayout_9.addWidget(self.lblTitulo)
        self.tab = QtWidgets.QTabWidget(frmInvestmentReport)
        self.tab.setEnabled(True)
        self.tab.setObjectName("tab")
        self.tab_12 = QtWidgets.QWidget()
        self.tab_12.setObjectName("tab_12")
        self.horizontalLayout_15 = QtWidgets.QHBoxLayout(self.tab_12)
        self.horizontalLayout_15.setObjectName("horizontalLayout_15")
        self.verticalLayout_4 = QtWidgets.QVBoxLayout()
        self.verticalLayout_4.setObjectName("verticalLayout_4")
        spacerItem = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout_4.addItem(spacerItem)
        self.horizontalLayout_12 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_12.setObjectName("horizontalLayout_12")
        spacerItem1 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_12.addItem(spacerItem1)
        self.groupBox = QtWidgets.QGroupBox(self.tab_12)
        self.groupBox.setTitle("")
        self.groupBox.setObjectName("groupBox")
        self.horizontalLayout_14 = QtWidgets.QHBoxLayout(self.groupBox)
        self.horizontalLayout_14.setObjectName("horizontalLayout_14")
        self.verticalLayout = QtWidgets.QVBoxLayout()
        self.verticalLayout.setObjectName("verticalLayout")
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.label = QtWidgets.QLabel(self.groupBox)
        self.label.setObjectName("label")
        self.horizontalLayout_2.addWidget(self.label)
        self.cmbAccount = QtWidgets.QComboBox(self.groupBox)
        self.cmbAccount.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.cmbAccount.setObjectName("cmbAccount")
        self.horizontalLayout_2.addWidget(self.cmbAccount)
        self.verticalLayout.addLayout(self.horizontalLayout_2)
        self.horizontalLayout_3 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_3.setObjectName("horizontalLayout_3")
        self.label_2 = QtWidgets.QLabel(self.groupBox)
        self.label_2.setObjectName("label_2")
        self.horizontalLayout_3.addWidget(self.label_2)
        self.txtInvestment = QtWidgets.QLineEdit(self.groupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.txtInvestment.sizePolicy().hasHeightForWidth())
        self.txtInvestment.setSizePolicy(sizePolicy)
        self.txtInvestment.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.txtInvestment.setObjectName("txtInvestment")
        self.horizontalLayout_3.addWidget(self.txtInvestment)
        self.verticalLayout.addLayout(self.horizontalLayout_3)
        self.horizontalLayout_8 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_8.setObjectName("horizontalLayout_8")
        self.ise = wdgProductSelector(self.groupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.ise.sizePolicy().hasHeightForWidth())
        self.ise.setSizePolicy(sizePolicy)
        self.ise.setObjectName("ise")
        self.horizontalLayout_8.addWidget(self.ise)
        self.verticalLayout.addLayout(self.horizontalLayout_8)
        self.horizontalLayout_5 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_5.setObjectName("horizontalLayout_5")
        self.label_4 = QtWidgets.QLabel(self.groupBox)
        self.label_4.setObjectName("label_4")
        self.horizontalLayout_5.addWidget(self.label_4)
        self.txtVenta = myQLineEdit(self.groupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.txtVenta.sizePolicy().hasHeightForWidth())
        self.txtVenta.setSizePolicy(sizePolicy)
        self.txtVenta.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.txtVenta.setObjectName("txtVenta")
        self.horizontalLayout_5.addWidget(self.txtVenta)
        self.cmdPuntoVenta = QtWidgets.QToolButton(self.groupBox)
        self.cmdPuntoVenta.setText("")
        icon1 = QtGui.QIcon()
        icon1.addPixmap(QtGui.QPixmap(":/xulpymoney/tools-wizard.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.cmdPuntoVenta.setIcon(icon1)
        self.cmdPuntoVenta.setObjectName("cmdPuntoVenta")
        self.horizontalLayout_5.addWidget(self.cmdPuntoVenta)
        self.verticalLayout.addLayout(self.horizontalLayout_5)
        self.horizontalLayout_10 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_10.setObjectName("horizontalLayout_10")
        self.chkExpiration = QtWidgets.QCheckBox(self.groupBox)
        self.chkExpiration.setChecked(True)
        self.chkExpiration.setObjectName("chkExpiration")
        self.horizontalLayout_10.addWidget(self.chkExpiration)
        self.cmdToday = QtWidgets.QToolButton(self.groupBox)
        self.cmdToday.setText("")
        icon2 = QtGui.QIcon()
        icon2.addPixmap(QtGui.QPixmap(":/xulpymoney/today.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.cmdToday.setIcon(icon2)
        self.cmdToday.setIconSize(QtCore.QSize(32, 32))
        self.cmdToday.setObjectName("cmdToday")
        self.horizontalLayout_10.addWidget(self.cmdToday)
        self.calExpiration = QtWidgets.QCalendarWidget(self.groupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.calExpiration.sizePolicy().hasHeightForWidth())
        self.calExpiration.setSizePolicy(sizePolicy)
        self.calExpiration.setFirstDayOfWeek(QtCore.Qt.Monday)
        self.calExpiration.setVerticalHeaderFormat(QtWidgets.QCalendarWidget.NoVerticalHeader)
        self.calExpiration.setObjectName("calExpiration")
        self.horizontalLayout_10.addWidget(self.calExpiration)
        self.verticalLayout.addLayout(self.horizontalLayout_10)
        self.cmdInvestment = QtWidgets.QPushButton(self.groupBox)
        self.cmdInvestment.setEnabled(False)
        icon3 = QtGui.QIcon()
        icon3.addPixmap(QtGui.QPixmap(":/xulpymoney/save.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.cmdInvestment.setIcon(icon3)
        self.cmdInvestment.setObjectName("cmdInvestment")
        self.verticalLayout.addWidget(self.cmdInvestment)
        self.horizontalLayout_14.addLayout(self.verticalLayout)
        self.horizontalLayout_12.addWidget(self.groupBox)
        spacerItem2 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_12.addItem(spacerItem2)
        self.verticalLayout_4.addLayout(self.horizontalLayout_12)
        spacerItem3 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout_4.addItem(spacerItem3)
        self.horizontalLayout_15.addLayout(self.verticalLayout_4)
        icon4 = QtGui.QIcon()
        icon4.addPixmap(QtGui.QPixmap(":/xulpymoney/bundle.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.tab.addTab(self.tab_12, icon4, "")
        self.tabInvestmentCurrent = QtWidgets.QWidget()
        self.tabInvestmentCurrent.setObjectName("tabInvestmentCurrent")
        self.horizontalLayout = QtWidgets.QHBoxLayout(self.tabInvestmentCurrent)
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.verticalLayout_11 = QtWidgets.QVBoxLayout()
        self.verticalLayout_11.setObjectName("verticalLayout_11")
        self.grpCurrentInvestmentCurrency = QtWidgets.QGroupBox(self.tabInvestmentCurrent)
        self.grpCurrentInvestmentCurrency.setObjectName("grpCurrentInvestmentCurrency")
        self.verticalLayout_6 = QtWidgets.QVBoxLayout(self.grpCurrentInvestmentCurrency)
        self.verticalLayout_6.setObjectName("verticalLayout_6")
        self.tblInvestmentCurrent = myQTableWidget(self.grpCurrentInvestmentCurrency)
        self.tblInvestmentCurrent.setContextMenuPolicy(QtCore.Qt.CustomContextMenu)
        self.tblInvestmentCurrent.setAlternatingRowColors(True)
        self.tblInvestmentCurrent.setSelectionMode(QtWidgets.QAbstractItemView.SingleSelection)
        self.tblInvestmentCurrent.setSelectionBehavior(QtWidgets.QAbstractItemView.SelectRows)
        self.tblInvestmentCurrent.setObjectName("tblInvestmentCurrent")
        self.tblInvestmentCurrent.setColumnCount(0)
        self.tblInvestmentCurrent.setRowCount(0)
        self.tblInvestmentCurrent.verticalHeader().setVisible(False)
        self.verticalLayout_6.addWidget(self.tblInvestmentCurrent)
        self.verticalLayout_11.addWidget(self.grpCurrentInvestmentCurrency)
        self.grpCurrentAccountCurrency = QtWidgets.QGroupBox(self.tabInvestmentCurrent)
        self.grpCurrentAccountCurrency.setObjectName("grpCurrentAccountCurrency")
        self.verticalLayout_8 = QtWidgets.QVBoxLayout(self.grpCurrentAccountCurrency)
        self.verticalLayout_8.setObjectName("verticalLayout_8")
        self.tblInvestmentCurrentAccountCurrency = myQTableWidget(self.grpCurrentAccountCurrency)
        self.tblInvestmentCurrentAccountCurrency.setContextMenuPolicy(QtCore.Qt.CustomContextMenu)
        self.tblInvestmentCurrentAccountCurrency.setAlternatingRowColors(True)
        self.tblInvestmentCurrentAccountCurrency.setSelectionMode(QtWidgets.QAbstractItemView.SingleSelection)
        self.tblInvestmentCurrentAccountCurrency.setSelectionBehavior(QtWidgets.QAbstractItemView.SelectRows)
        self.tblInvestmentCurrentAccountCurrency.setObjectName("tblInvestmentCurrentAccountCurrency")
        self.tblInvestmentCurrentAccountCurrency.setColumnCount(0)
        self.tblInvestmentCurrentAccountCurrency.setRowCount(0)
        self.tblInvestmentCurrentAccountCurrency.verticalHeader().setVisible(False)
        self.verticalLayout_8.addWidget(self.tblInvestmentCurrentAccountCurrency)
        self.verticalLayout_11.addWidget(self.grpCurrentAccountCurrency)
        self.lblAge = QtWidgets.QLabel(self.tabInvestmentCurrent)
        font = QtGui.QFont()
        font.setPointSize(8)
        font.setBold(True)
        font.setWeight(75)
        self.lblAge.setFont(font)
        self.lblAge.setAlignment(QtCore.Qt.AlignCenter)
        self.lblAge.setObjectName("lblAge")
        self.verticalLayout_11.addWidget(self.lblAge)
        self.horizontalLayout.addLayout(self.verticalLayout_11)
        icon5 = QtGui.QIcon()
        icon5.addPixmap(QtGui.QPixmap(":/xulpymoney/document-edit.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.tab.addTab(self.tabInvestmentCurrent, icon5, "")
        self.tabOperacionesHistoricas = QtWidgets.QWidget()
        self.tabOperacionesHistoricas.setObjectName("tabOperacionesHistoricas")
        self.horizontalLayout_17 = QtWidgets.QHBoxLayout(self.tabOperacionesHistoricas)
        self.horizontalLayout_17.setObjectName("horizontalLayout_17")
        self.verticalLayout_7 = QtWidgets.QVBoxLayout()
        self.verticalLayout_7.setObjectName("verticalLayout_7")
        self.chkOperaciones = QtWidgets.QCheckBox(self.tabOperacionesHistoricas)
        self.chkOperaciones.setObjectName("chkOperaciones")
        self.verticalLayout_7.addWidget(self.chkOperaciones)
        self.grpOperationsInvestmentCurrency = QtWidgets.QGroupBox(self.tabOperacionesHistoricas)
        self.grpOperationsInvestmentCurrency.setObjectName("grpOperationsInvestmentCurrency")
        self.horizontalLayout_9 = QtWidgets.QHBoxLayout(self.grpOperationsInvestmentCurrency)
        self.horizontalLayout_9.setObjectName("horizontalLayout_9")
        self.tblOperations = myQTableWidget(self.grpOperationsInvestmentCurrency)
        self.tblOperations.setContextMenuPolicy(QtCore.Qt.CustomContextMenu)
        self.tblOperations.setAlternatingRowColors(True)
        self.tblOperations.setSelectionMode(QtWidgets.QAbstractItemView.SingleSelection)
        self.tblOperations.setSelectionBehavior(QtWidgets.QAbstractItemView.SelectRows)
        self.tblOperations.setObjectName("tblOperations")
        self.tblOperations.setColumnCount(0)
        self.tblOperations.setRowCount(0)
        self.tblOperations.verticalHeader().setVisible(False)
        self.horizontalLayout_9.addWidget(self.tblOperations)
        self.verticalLayout_7.addWidget(self.grpOperationsInvestmentCurrency)
        self.grpOperationsAccountCurrency = QtWidgets.QGroupBox(self.tabOperacionesHistoricas)
        self.grpOperationsAccountCurrency.setObjectName("grpOperationsAccountCurrency")
        self.horizontalLayout_16 = QtWidgets.QHBoxLayout(self.grpOperationsAccountCurrency)
        self.horizontalLayout_16.setObjectName("horizontalLayout_16")
        self.tblOperationsAccountCurrency = myQTableWidget(self.grpOperationsAccountCurrency)
        self.tblOperationsAccountCurrency.setContextMenuPolicy(QtCore.Qt.CustomContextMenu)
        self.tblOperationsAccountCurrency.setAlternatingRowColors(True)
        self.tblOperationsAccountCurrency.setSelectionMode(QtWidgets.QAbstractItemView.SingleSelection)
        self.tblOperationsAccountCurrency.setSelectionBehavior(QtWidgets.QAbstractItemView.SelectRows)
        self.tblOperationsAccountCurrency.setObjectName("tblOperationsAccountCurrency")
        self.tblOperationsAccountCurrency.setColumnCount(0)
        self.tblOperationsAccountCurrency.setRowCount(0)
        self.tblOperationsAccountCurrency.verticalHeader().setVisible(False)
        self.horizontalLayout_16.addWidget(self.tblOperationsAccountCurrency)
        self.verticalLayout_7.addWidget(self.grpOperationsAccountCurrency)
        self.horizontalLayout_17.addLayout(self.verticalLayout_7)
        self.tab.addTab(self.tabOperacionesHistoricas, icon5, "")
        self.tab_2 = QtWidgets.QWidget()
        self.tab_2.setObjectName("tab_2")
        self.horizontalLayout_4 = QtWidgets.QHBoxLayout(self.tab_2)
        self.horizontalLayout_4.setObjectName("horizontalLayout_4")
        self.verticalLayout_14 = QtWidgets.QVBoxLayout()
        self.verticalLayout_14.setObjectName("verticalLayout_14")
        self.grpHistoricalInvestmentCurrency = QtWidgets.QGroupBox(self.tab_2)
        self.grpHistoricalInvestmentCurrency.setObjectName("grpHistoricalInvestmentCurrency")
        self.verticalLayout_12 = QtWidgets.QVBoxLayout(self.grpHistoricalInvestmentCurrency)
        self.verticalLayout_12.setObjectName("verticalLayout_12")
        self.tblInvestmentHistorical = myQTableWidget(self.grpHistoricalInvestmentCurrency)
        self.tblInvestmentHistorical.setContextMenuPolicy(QtCore.Qt.CustomContextMenu)
        self.tblInvestmentHistorical.setEditTriggers(QtWidgets.QAbstractItemView.NoEditTriggers)
        self.tblInvestmentHistorical.setAlternatingRowColors(True)
        self.tblInvestmentHistorical.setSelectionMode(QtWidgets.QAbstractItemView.NoSelection)
        self.tblInvestmentHistorical.setObjectName("tblInvestmentHistorical")
        self.tblInvestmentHistorical.setColumnCount(0)
        self.tblInvestmentHistorical.setRowCount(0)
        self.tblInvestmentHistorical.verticalHeader().setVisible(False)
        self.verticalLayout_12.addWidget(self.tblInvestmentHistorical)
        self.verticalLayout_14.addWidget(self.grpHistoricalInvestmentCurrency)
        self.grpHistoricalAccountCurrency = QtWidgets.QGroupBox(self.tab_2)
        self.grpHistoricalAccountCurrency.setObjectName("grpHistoricalAccountCurrency")
        self.verticalLayout_13 = QtWidgets.QVBoxLayout(self.grpHistoricalAccountCurrency)
        self.verticalLayout_13.setObjectName("verticalLayout_13")
        self.tblInvestmentHistoricalAccountCurrency = myQTableWidget(self.grpHistoricalAccountCurrency)
        self.tblInvestmentHistoricalAccountCurrency.setContextMenuPolicy(QtCore.Qt.CustomContextMenu)
        self.tblInvestmentHistoricalAccountCurrency.setEditTriggers(QtWidgets.QAbstractItemView.NoEditTriggers)
        self.tblInvestmentHistoricalAccountCurrency.setAlternatingRowColors(True)
        self.tblInvestmentHistoricalAccountCurrency.setSelectionMode(QtWidgets.QAbstractItemView.NoSelection)
        self.tblInvestmentHistoricalAccountCurrency.setObjectName("tblInvestmentHistoricalAccountCurrency")
        self.tblInvestmentHistoricalAccountCurrency.setColumnCount(0)
        self.tblInvestmentHistoricalAccountCurrency.setRowCount(0)
        self.tblInvestmentHistoricalAccountCurrency.verticalHeader().setVisible(False)
        self.verticalLayout_13.addWidget(self.tblInvestmentHistoricalAccountCurrency)
        self.verticalLayout_14.addWidget(self.grpHistoricalAccountCurrency)
        self.horizontalLayout_4.addLayout(self.verticalLayout_14)
        icon6 = QtGui.QIcon()
        icon6.addPixmap(QtGui.QPixmap(":/xulpymoney/history2.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.tab.addTab(self.tab_2, icon6, "")
        self.tabDividends = QtWidgets.QWidget()
        self.tabDividends.setObjectName("tabDividends")
        self.horizontalLayout_7 = QtWidgets.QHBoxLayout(self.tabDividends)
        self.horizontalLayout_7.setObjectName("horizontalLayout_7")
        self.verticalLayout_16 = QtWidgets.QVBoxLayout()
        self.verticalLayout_16.setObjectName("verticalLayout_16")
        self.chkHistoricalDividends = QtWidgets.QCheckBox(self.tabDividends)
        self.chkHistoricalDividends.setObjectName("chkHistoricalDividends")
        self.verticalLayout_16.addWidget(self.chkHistoricalDividends)
        self.grpDividendsInvestmentCurrency = QtWidgets.QGroupBox(self.tabDividends)
        self.grpDividendsInvestmentCurrency.setObjectName("grpDividendsInvestmentCurrency")
        self.verticalLayout_5 = QtWidgets.QVBoxLayout(self.grpDividendsInvestmentCurrency)
        self.verticalLayout_5.setObjectName("verticalLayout_5")
        self.tblDividends = myQTableWidget(self.grpDividendsInvestmentCurrency)
        self.tblDividends.setContextMenuPolicy(QtCore.Qt.CustomContextMenu)
        self.tblDividends.setAlternatingRowColors(True)
        self.tblDividends.setSelectionMode(QtWidgets.QAbstractItemView.SingleSelection)
        self.tblDividends.setSelectionBehavior(QtWidgets.QAbstractItemView.SelectRows)
        self.tblDividends.setObjectName("tblDividends")
        self.tblDividends.setColumnCount(0)
        self.tblDividends.setRowCount(0)
        self.tblDividends.verticalHeader().setVisible(False)
        self.verticalLayout_5.addWidget(self.tblDividends)
        self.verticalLayout_16.addWidget(self.grpDividendsInvestmentCurrency)
        self.grpDividendsAccountCurrency = QtWidgets.QGroupBox(self.tabDividends)
        self.grpDividendsAccountCurrency.setObjectName("grpDividendsAccountCurrency")
        self.verticalLayout_15 = QtWidgets.QVBoxLayout(self.grpDividendsAccountCurrency)
        self.verticalLayout_15.setObjectName("verticalLayout_15")
        self.tblDividendsAccountCurrency = myQTableWidget(self.grpDividendsAccountCurrency)
        self.tblDividendsAccountCurrency.setContextMenuPolicy(QtCore.Qt.CustomContextMenu)
        self.tblDividendsAccountCurrency.setAlternatingRowColors(True)
        self.tblDividendsAccountCurrency.setSelectionMode(QtWidgets.QAbstractItemView.SingleSelection)
        self.tblDividendsAccountCurrency.setSelectionBehavior(QtWidgets.QAbstractItemView.SelectRows)
        self.tblDividendsAccountCurrency.setObjectName("tblDividendsAccountCurrency")
        self.tblDividendsAccountCurrency.setColumnCount(0)
        self.tblDividendsAccountCurrency.setRowCount(0)
        self.tblDividendsAccountCurrency.verticalHeader().setVisible(False)
        self.verticalLayout_15.addWidget(self.tblDividendsAccountCurrency)
        self.verticalLayout_16.addWidget(self.grpDividendsAccountCurrency)
        self.horizontalLayout_6 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_6.setObjectName("horizontalLayout_6")
        self.grpDividendsEstimation = QtWidgets.QGroupBox(self.tabDividends)
        self.grpDividendsEstimation.setObjectName("grpDividendsEstimation")
        self.horizontalLayout_11 = QtWidgets.QHBoxLayout(self.grpDividendsEstimation)
        self.horizontalLayout_11.setObjectName("horizontalLayout_11")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout()
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.lblDivAnualEstimado = QtWidgets.QLabel(self.grpDividendsEstimation)
        self.lblDivAnualEstimado.setObjectName("lblDivAnualEstimado")
        self.verticalLayout_2.addWidget(self.lblDivAnualEstimado)
        self.lblDivFechaRevision = QtWidgets.QLabel(self.grpDividendsEstimation)
        self.lblDivFechaRevision.setObjectName("lblDivFechaRevision")
        self.verticalLayout_2.addWidget(self.lblDivFechaRevision)
        self.lblDivSaldoEstimado = QtWidgets.QLabel(self.grpDividendsEstimation)
        self.lblDivSaldoEstimado.setObjectName("lblDivSaldoEstimado")
        self.verticalLayout_2.addWidget(self.lblDivSaldoEstimado)
        self.horizontalLayout_11.addLayout(self.verticalLayout_2)
        self.horizontalLayout_6.addWidget(self.grpDividendsEstimation)
        self.grpDividendsEfectivos = QtWidgets.QGroupBox(self.tabDividends)
        self.grpDividendsEfectivos.setObjectName("grpDividendsEfectivos")
        self.horizontalLayout_13 = QtWidgets.QHBoxLayout(self.grpDividendsEfectivos)
        self.horizontalLayout_13.setObjectName("horizontalLayout_13")
        self.verticalLayout_3 = QtWidgets.QVBoxLayout()
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.lblDivTPC = QtWidgets.QLabel(self.grpDividendsEfectivos)
        self.lblDivTPC.setObjectName("lblDivTPC")
        self.verticalLayout_3.addWidget(self.lblDivTPC)
        self.lblDivTAE = QtWidgets.QLabel(self.grpDividendsEfectivos)
        self.lblDivTAE.setObjectName("lblDivTAE")
        self.verticalLayout_3.addWidget(self.lblDivTAE)
        self.horizontalLayout_13.addLayout(self.verticalLayout_3)
        self.horizontalLayout_6.addWidget(self.grpDividendsEfectivos)
        self.verticalLayout_16.addLayout(self.horizontalLayout_6)
        self.horizontalLayout_7.addLayout(self.verticalLayout_16)
        icon7 = QtGui.QIcon()
        icon7.addPixmap(QtGui.QPixmap(":/xulpymoney/dividends.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.tab.addTab(self.tabDividends, icon7, "")
        self.tabContracts = QtWidgets.QWidget()
        self.tabContracts.setObjectName("tabContracts")
        self.horizontalLayout_19 = QtWidgets.QHBoxLayout(self.tabContracts)
        self.horizontalLayout_19.setObjectName("horizontalLayout_19")
        self.verticalLayout_17 = QtWidgets.QVBoxLayout()
        self.verticalLayout_17.setObjectName("verticalLayout_17")
        self.chkHistoricalContracts = QtWidgets.QCheckBox(self.tabContracts)
        self.chkHistoricalContracts.setObjectName("chkHistoricalContracts")
        self.verticalLayout_17.addWidget(self.chkHistoricalContracts)
        self.grpHlContracts = QtWidgets.QGroupBox(self.tabContracts)
        self.grpHlContracts.setObjectName("grpHlContracts")
        self.horizontalLayout_18 = QtWidgets.QHBoxLayout(self.grpHlContracts)
        self.horizontalLayout_18.setObjectName("horizontalLayout_18")
        self.tblHlContracts = myQTableWidget(self.grpHlContracts)
        self.tblHlContracts.setContextMenuPolicy(QtCore.Qt.CustomContextMenu)
        self.tblHlContracts.setEditTriggers(QtWidgets.QAbstractItemView.NoEditTriggers)
        self.tblHlContracts.setAlternatingRowColors(True)
        self.tblHlContracts.setSelectionMode(QtWidgets.QAbstractItemView.SingleSelection)
        self.tblHlContracts.setSelectionBehavior(QtWidgets.QAbstractItemView.SelectRows)
        self.tblHlContracts.setObjectName("tblHlContracts")
        self.tblHlContracts.setColumnCount(0)
        self.tblHlContracts.setRowCount(0)
        self.tblHlContracts.verticalHeader().setVisible(False)
        self.horizontalLayout_18.addWidget(self.tblHlContracts)
        self.verticalLayout_17.addWidget(self.grpHlContracts)
        self.horizontalLayout_19.addLayout(self.verticalLayout_17)
        self.tab.addTab(self.tabContracts, icon6, "")
        self.verticalLayout_9.addWidget(self.tab)
        self.verticalLayout_10.addLayout(self.verticalLayout_9)
        self.actionOperationAdd = QtWidgets.QAction(frmInvestmentReport)
        icon8 = QtGui.QIcon()
        icon8.addPixmap(QtGui.QPixmap(":/xulpymoney/add.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.actionOperationAdd.setIcon(icon8)
        self.actionOperationAdd.setObjectName("actionOperationAdd")
        self.actionOperationDelete = QtWidgets.QAction(frmInvestmentReport)
        icon9 = QtGui.QIcon()
        icon9.addPixmap(QtGui.QPixmap(":/xulpymoney/list-remove.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.actionOperationDelete.setIcon(icon9)
        self.actionOperationDelete.setObjectName("actionOperationDelete")
        self.actionDividendAdd = QtWidgets.QAction(frmInvestmentReport)
        self.actionDividendAdd.setIcon(icon8)
        self.actionDividendAdd.setObjectName("actionDividendAdd")
        self.actionDividendRemove = QtWidgets.QAction(frmInvestmentReport)
        self.actionDividendRemove.setIcon(icon9)
        self.actionDividendRemove.setObjectName("actionDividendRemove")
        self.actionOperationEdit = QtWidgets.QAction(frmInvestmentReport)
        icon10 = QtGui.QIcon()
        icon10.addPixmap(QtGui.QPixmap(":/xulpymoney/editar.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.actionOperationEdit.setIcon(icon10)
        self.actionOperationEdit.setObjectName("actionOperationEdit")
        self.actionDisReinvest = QtWidgets.QAction(frmInvestmentReport)
        self.actionDisReinvest.setIcon(icon6)
        self.actionDisReinvest.setObjectName("actionDisReinvest")
        self.actionSellingPoint = QtWidgets.QAction(frmInvestmentReport)
        self.actionSellingPoint.setIcon(icon1)
        self.actionSellingPoint.setObjectName("actionSellingPoint")
        self.actionDividendEdit = QtWidgets.QAction(frmInvestmentReport)
        self.actionDividendEdit.setIcon(icon10)
        self.actionDividendEdit.setObjectName("actionDividendEdit")
        self.actionSharesTransferUndo = QtWidgets.QAction(frmInvestmentReport)
        self.actionSharesTransferUndo.setIcon(icon9)
        self.actionSharesTransferUndo.setObjectName("actionSharesTransferUndo")
        self.actionSharesTransfer = QtWidgets.QAction(frmInvestmentReport)
        icon11 = QtGui.QIcon()
        icon11.addPixmap(QtGui.QPixmap(":/xulpymoney/transfer.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.actionSharesTransfer.setIcon(icon11)
        self.actionSharesTransfer.setObjectName("actionSharesTransfer")
        self.actionSplit = QtWidgets.QAction(frmInvestmentReport)
        self.actionSplit.setIcon(icon10)
        self.actionSplit.setObjectName("actionSplit")
        self.actionChangeBenchmarkPrice = QtWidgets.QAction(frmInvestmentReport)
        self.actionChangeBenchmarkPrice.setIcon(icon10)
        self.actionChangeBenchmarkPrice.setObjectName("actionChangeBenchmarkPrice")
        self.actionRangeReport = QtWidgets.QAction(frmInvestmentReport)
        icon12 = QtGui.QIcon()
        icon12.addPixmap(QtGui.QPixmap(":/xulpymoney/eye.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.actionRangeReport.setIcon(icon12)
        self.actionRangeReport.setObjectName("actionRangeReport")
        self.actionDisReinvestProduct = QtWidgets.QAction(frmInvestmentReport)
        icon13 = QtGui.QIcon()
        icon13.addPixmap(QtGui.QPixmap(":/xulpymoney/study.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.actionDisReinvestProduct.setIcon(icon13)
        self.actionDisReinvestProduct.setObjectName("actionDisReinvestProduct")
        self.actionContractAdd = QtWidgets.QAction(frmInvestmentReport)
        self.actionContractAdd.setIcon(icon8)
        self.actionContractAdd.setObjectName("actionContractAdd")
        self.actionContractDelete = QtWidgets.QAction(frmInvestmentReport)
        self.actionContractDelete.setIcon(icon9)
        self.actionContractDelete.setObjectName("actionContractDelete")
        self.actionContractEdit = QtWidgets.QAction(frmInvestmentReport)
        self.actionContractEdit.setIcon(icon10)
        self.actionContractEdit.setObjectName("actionContractEdit")

        self.retranslateUi(frmInvestmentReport)
        self.tab.setCurrentIndex(5)
        self.chkExpiration.toggled['bool'].connect(self.calExpiration.setEnabled)
        QtCore.QMetaObject.connectSlotsByName(frmInvestmentReport)

    def retranslateUi(self, frmInvestmentReport):
        _translate = QtCore.QCoreApplication.translate
        frmInvestmentReport.setWindowTitle(_translate("frmInvestmentReport", "Investment report"))
        self.label.setText(_translate("frmInvestmentReport", "Associated account"))
        self.label_2.setText(_translate("frmInvestmentReport", "Investment name"))
        self.txtInvestment.setText(_translate("frmInvestmentReport", "New investment"))
        self.label_4.setText(_translate("frmInvestmentReport", "Selling price"))
        self.txtVenta.setText(_translate("frmInvestmentReport", "0"))
        self.chkExpiration.setText(_translate("frmInvestmentReport", "Add selling order expiration"))
        self.cmdToday.setToolTip(_translate("frmInvestmentReport", "Change calendar date to today"))
        self.cmdInvestment.setText(_translate("frmInvestmentReport", "Update investment"))
        self.tab.setTabText(self.tab.indexOf(self.tab_12), _translate("frmInvestmentReport", "Investment data"))
        self.grpCurrentInvestmentCurrency.setTitle(_translate("frmInvestmentReport", "Current status in investment currency"))
        self.grpCurrentAccountCurrency.setTitle(_translate("frmInvestmentReport", "Current status in account currency"))
        self.tab.setTabText(self.tab.indexOf(self.tabInvestmentCurrent), _translate("frmInvestmentReport", "Current status"))
        self.chkOperaciones.setText(_translate("frmInvestmentReport", "Show all operations"))
        self.grpOperationsInvestmentCurrency.setTitle(_translate("frmInvestmentReport", "Operations in investment currency"))
        self.grpOperationsAccountCurrency.setTitle(_translate("frmInvestmentReport", "Operations in account currency"))
        self.tab.setTabText(self.tab.indexOf(self.tabOperacionesHistoricas), _translate("frmInvestmentReport", "Operations"))
        self.grpHistoricalInvestmentCurrency.setTitle(_translate("frmInvestmentReport", "Historical operations in investment currency"))
        self.grpHistoricalAccountCurrency.setTitle(_translate("frmInvestmentReport", "Historical operations in account currency"))
        self.tab.setTabText(self.tab.indexOf(self.tab_2), _translate("frmInvestmentReport", "Historical operations"))
        self.chkHistoricalDividends.setText(_translate("frmInvestmentReport", "Show historical dividends"))
        self.grpDividendsInvestmentCurrency.setTitle(_translate("frmInvestmentReport", "Dividends in investment currency"))
        self.grpDividendsAccountCurrency.setTitle(_translate("frmInvestmentReport", "Dividends in account currency"))
        self.grpDividendsEstimation.setTitle(_translate("frmInvestmentReport", "Estimation"))
        self.lblDivAnualEstimado.setText(_translate("frmInvestmentReport", "APR"))
        self.lblDivFechaRevision.setText(_translate("frmInvestmentReport", "Review date"))
        self.lblDivSaldoEstimado.setText(_translate("frmInvestmentReport", "Estimated balance"))
        self.grpDividendsEfectivos.setTitle(_translate("frmInvestmentReport", "Collected dividends"))
        self.lblDivTPC.setText(_translate("frmInvestmentReport", "% Invested"))
        self.lblDivTAE.setText(_translate("frmInvestmentReport", "% APR"))
        self.tab.setTabText(self.tab.indexOf(self.tabDividends), _translate("frmInvestmentReport", "Dividends"))
        self.chkHistoricalContracts.setText(_translate("frmInvestmentReport", "Show historical High-Low daily contracts"))
        self.grpHlContracts.setTitle(_translate("frmInvestmentReport", "High-Low operations daily contracts"))
        self.tab.setTabText(self.tab.indexOf(self.tabContracts), _translate("frmInvestmentReport", "Contracts"))
        self.actionOperationAdd.setText(_translate("frmInvestmentReport", "New operation"))
        self.actionOperationAdd.setToolTip(_translate("frmInvestmentReport", "New operation"))
        self.actionOperationDelete.setText(_translate("frmInvestmentReport", "Delete operation"))
        self.actionOperationDelete.setToolTip(_translate("frmInvestmentReport", "Delete operation"))
        self.actionDividendAdd.setText(_translate("frmInvestmentReport", "Add new dividend"))
        self.actionDividendAdd.setToolTip(_translate("frmInvestmentReport", "Add new dividend"))
        self.actionDividendRemove.setText(_translate("frmInvestmentReport", "Delete dividend"))
        self.actionDividendRemove.setToolTip(_translate("frmInvestmentReport", "Delete dividend"))
        self.actionOperationEdit.setText(_translate("frmInvestmentReport", "Edit operation"))
        self.actionOperationEdit.setToolTip(_translate("frmInvestmentReport", "Edit operation"))
        self.actionDisReinvest.setText(_translate("frmInvestmentReport", "Disinvest / Reinvest report"))
        self.actionDisReinvest.setToolTip(_translate("frmInvestmentReport", "Disinvest / Reinvest report"))
        self.actionSellingPoint.setText(_translate("frmInvestmentReport", "Selling point calculation"))
        self.actionSellingPoint.setToolTip(_translate("frmInvestmentReport", "Selling point calculation"))
        self.actionDividendEdit.setText(_translate("frmInvestmentReport", "Edit dividend"))
        self.actionDividendEdit.setToolTip(_translate("frmInvestmentReport", "Edit dividend"))
        self.actionSharesTransferUndo.setText(_translate("frmInvestmentReport", "Undo shares transfer to another bank"))
        self.actionSharesTransferUndo.setToolTip(_translate("frmInvestmentReport", "Undo shares transfer to another bank"))
        self.actionSharesTransfer.setText(_translate("frmInvestmentReport", "Transfer shares to another bank"))
        self.actionSharesTransfer.setToolTip(_translate("frmInvestmentReport", "Transfer shares to another bank"))
        self.actionSplit.setText(_translate("frmInvestmentReport", "Make Split / Contrasplit"))
        self.actionSplit.setToolTip(_translate("frmInvestmentReport", "make Split / Contrasplit"))
        self.actionChangeBenchmarkPrice.setText(_translate("frmInvestmentReport", "Change benchmark price"))
        self.actionChangeBenchmarkPrice.setToolTip(_translate("frmInvestmentReport", "Change benchmark price"))
        self.actionRangeReport.setText(_translate("frmInvestmentReport", "Hide in range report"))
        self.actionRangeReport.setToolTip(_translate("frmInvestmentReport", "Hide in range report"))
        self.actionDisReinvestProduct.setText(_translate("frmInvestmentReport", "Disinvest / Reinvest all product"))
        self.actionDisReinvestProduct.setToolTip(_translate("frmInvestmentReport", "Disinvest / Reinvest all product"))
        self.actionContractAdd.setText(_translate("frmInvestmentReport", "New High-Low contract"))
        self.actionContractAdd.setToolTip(_translate("frmInvestmentReport", "New High-Low contract"))
        self.actionContractDelete.setText(_translate("frmInvestmentReport", "Delete contract"))
        self.actionContractDelete.setToolTip(_translate("frmInvestmentReport", "Delete contract"))
        self.actionContractEdit.setText(_translate("frmInvestmentReport", "Edit contract"))
        self.actionContractEdit.setToolTip(_translate("frmInvestmentReport", "Edit contract"))


from xulpymoney.ui.myqlineedit import myQLineEdit
from xulpymoney.ui.myqtablewidget import myQTableWidget
from xulpymoney.ui.wdgProductSelector import wdgProductSelector
import xulpymoney.images.xulpymoney_rc



import datetime
import unittest
import logging
from decimal import Decimal
from xulpymoney.libxulpymoney import Investment, MemXulpymoney, InvestmentOperation, InvestmentOperationHomogeneusManager, OperationType, OperationTypeManager, Product
from xulpymoney.libxulpymoneytypes import eOperationType

class TestXulpymoneyData(unittest.TestCase):
#    def setUp(self):
#        self.app = app
#        self.mem=mem
#        self.frmMain=frmMain 

    def test_actualizar(self):
        mem=MemXulpymoney()
        account=None
        product=Product(mem)
        product.high_low=True
        tiposoperaciones=OperationTypeManager(mem)
        tiposoperaciones.append(OperationType().init__create( "Purchase of shares", eOperationType.SharesPurchase))
        tiposoperaciones.append(OperationType().init__create( "Sale of shares", eOperationType.SharesSale))
        tiposoperaciones.append(OperationType().init__create( "Added of shares", eOperationType.SharesAdd))
        inv=Investment(mem).init__create("CDF Ibex", None, account,  product, None, True, 1)

        #Compro 0.1 y 0.1 y Vendo 0.1
        inv.op=InvestmentOperationHomogeneusManager(mem, inv)
        inv.op.append(InvestmentOperation(mem).init__create(tiposoperaciones.find_by_id(eOperationType.SharesPurchase), datetime.datetime.now(), inv,  Decimal("0.1"), 0, 0, 8000, "", True, 1, 1 ))
        inv.op.append(InvestmentOperation(mem).init__create(tiposoperaciones.find_by_id(eOperationType.SharesPurchase), datetime.datetime.now(), inv,  Decimal("0.1"), 0, 0, 8000, "", True, 1, 1 ))
        inv.op.append(InvestmentOperation(mem).init__create(tiposoperaciones.find_by_id(eOperationType.SharesSale), datetime.datetime.now(), inv,  Decimal("-0.1"), 0, 0, 8000, "", True, 1, 1 ))
        inv.op_actual, inv.op_historica = inv.op.get_current_and_historical_operations(test_suite=True)
        self.assertEqual(Decimal("0.1"), inv.op_actual.shares())
        
        #Vendo -0.1 y -0.1 y Compro 0.1
        inv.op=InvestmentOperationHomogeneusManager(mem, inv)
        inv.op.append(InvestmentOperation(mem).init__create(tiposoperaciones.find_by_id(eOperationType.SharesSale), datetime.datetime.now(), inv,  Decimal("-0.1"), 0, 0, 8000, "", True, 1, 1 ))
        inv.op.append(InvestmentOperation(mem).init__create(tiposoperaciones.find_by_id(eOperationType.SharesSale), datetime.datetime.now(), inv,  Decimal("-0.1"), 0, 0, 8000, "", True, 1, 1 ))
        inv.op.append(InvestmentOperation(mem).init__create(tiposoperaciones.find_by_id(eOperationType.SharesPurchase), datetime.datetime.now(), inv,  Decimal("+0.1"), 0, 0, 8000, "", True, 1, 1 ))
        inv.op_actual, inv.op_historica = inv.op.get_current_and_historical_operations(test_suite=True)
        self.assertEqual(Decimal("-0.1"), inv.op_actual.shares())
        
        #Compro 0.3 y 0.2 y Vemdo 0.7
        inv.op=InvestmentOperationHomogeneusManager(mem, inv)
        inv.op.append(InvestmentOperation(mem).init__create(tiposoperaciones.find_by_id(eOperationType.SharesPurchase), datetime.datetime.now(), inv,  Decimal("0.3"), 0, 0, 8000, "", True, 1, 1 ))
        inv.op.append(InvestmentOperation(mem).init__create(tiposoperaciones.find_by_id(eOperationType.SharesPurchase), datetime.datetime.now(), inv,  Decimal("0.2"), 0, 0, 8000, "", True, 1, 1 ))
        inv.op.append(InvestmentOperation(mem).init__create(tiposoperaciones.find_by_id(eOperationType.SharesSale), datetime.datetime.now(), inv,  Decimal("-0.7"), 0, 0, 8000, "", True, 1, 1 ))
        inv.op.append(InvestmentOperation(mem).init__create(tiposoperaciones.find_by_id(eOperationType.SharesPurchase), datetime.datetime.now(), inv,  Decimal("0.1"), 0, 0, 8000, "", True, 1, 1 ))
        inv.op_actual, inv.op_historica = inv.op.get_current_and_historical_operations(test_suite=True)
        self.assertEqual(Decimal("-0.1"), inv.op_actual.shares())
        self.assertEqual(1, inv.op_actual.length())

if __name__ == '__main__':
    
    logFormat = "%(asctime)s.%(msecs)03d %(levelname)s %(message)s [%(module)s:%(lineno)d]"
    dateFormat='%F %I:%M:%S'
    logging.basicConfig(level=logging.DEBUG, format=logFormat, datefmt=dateFormat)

    unittest.main()



import datetime
from PyQt5.QtCore import pyqtSlot
from PyQt5.QtWidgets import QWidget
from xulpymoney.libxulpymoney import Order
from xulpymoney.libxulpymoneyfunctions import qmessagebox
from xulpymoney.ui.Ui_wdgOrdersAdd import Ui_wdgOrdersAdd

class wdgOrdersAdd(QWidget, Ui_wdgOrdersAdd):
    def __init__(self, mem, order=None, investment=None,   parent=None):
        QWidget.__init__(self, parent)
        self.setupUi(self)
        self.mem=mem
        self.order=order
        self.parent=parent
        self.investment=investment

        if order==None:
            self.lbl.setText("Add new order")
            self.deDate.setDate(datetime.date.today())
            self.deExpiration.setDate(datetime.date.today())
        else:
            self.lbl.setText("Edit order")
            self.deDate.setDate(self.order.date)
            self.deExpiration.setDate(self.order.expiration)
            self.txtShares.setText(self.order.shares)
            self.txtAmount.setText(self.order.amount)
            self.txtPrice.setText(self.order.price)
            
        self.mem.data.investments.qcombobox(self.cmbInvestments, 2, selected=None, obsolete_product=False, investments_active=None,  accounts_active=True)
        if self.investment!=None:
            self.cmbInvestments.setCurrentIndex(self.cmbInvestments.findData(self.investment.id))
        else:
            self.cmbInvestments.setCurrentIndex(self.cmbInvestments.findData(-1))
            
        
    @pyqtSlot()
    def on_buttonbox_accepted(self):
        self.date=self.deDate.date()
        if not (self.txtAmount.isValid() and self.txtPrice.isValid() and self.txtShares.isValid()):
            qmessagebox(self.tr("Incorrect data. Try again."))
            return
        investment=self.mem.data.investments.find_by_id(self.cmbInvestments.itemData(self.cmbInvestments.currentIndex()))
        if investment==None:
            qmessagebox(self.tr("You must select an investment"))
            return
        if self.deExpiration.date()<self.deDate.date():
            qmessagebox(self.tr("Expiration date can't be less than order date."))
            return
            
        if self.order==None:
            self.order=Order(self.mem)
        self.order.date=self.deDate.date().toPyDate()
        self.order.expiration=self.deExpiration.date().toPyDate()
        self.order.shares=self.txtShares.decimal()
        self.order.amount=self.txtAmount.decimal()
        self.order.price=self.txtPrice.decimal()
        self.order.investment=investment
        
        self.order.save()
        self.mem.con.commit()
        self.order.qmessagebox_reminder()
        self.parent.accept()

    @pyqtSlot()
    def on_buttonbox_rejected(self):
        self.parent.reject()



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'xulpymoney/ui/wdgYearMonth.ui'
#
# Created by: PyQt5 UI code generator 5.12.2
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_wdgYearMonth(object):
    def setupUi(self, wdgYearMonth):
        wdgYearMonth.setObjectName("wdgYearMonth")
        wdgYearMonth.resize(673, 38)
        self.verticalLayout = QtWidgets.QVBoxLayout(wdgYearMonth)
        self.verticalLayout.setObjectName("verticalLayout")
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.label = QtWidgets.QLabel(wdgYearMonth)
        self.label.setObjectName("label")
        self.horizontalLayout.addWidget(self.label)
        self.cmdPrevious = QtWidgets.QToolButton(wdgYearMonth)
        self.cmdPrevious.setObjectName("cmdPrevious")
        self.horizontalLayout.addWidget(self.cmdPrevious)
        self.cmbMonth = QtWidgets.QComboBox(wdgYearMonth)
        self.cmbMonth.setObjectName("cmbMonth")
        self.cmbMonth.addItem("")
        self.cmbMonth.addItem("")
        self.cmbMonth.addItem("")
        self.cmbMonth.addItem("")
        self.cmbMonth.addItem("")
        self.cmbMonth.addItem("")
        self.cmbMonth.addItem("")
        self.cmbMonth.addItem("")
        self.cmbMonth.addItem("")
        self.cmbMonth.addItem("")
        self.cmbMonth.addItem("")
        self.cmbMonth.addItem("")
        self.horizontalLayout.addWidget(self.cmbMonth)
        self.cmbYear = QtWidgets.QComboBox(wdgYearMonth)
        self.cmbYear.setObjectName("cmbYear")
        self.horizontalLayout.addWidget(self.cmbYear)
        self.cmdNext = QtWidgets.QToolButton(wdgYearMonth)
        self.cmdNext.setObjectName("cmdNext")
        self.horizontalLayout.addWidget(self.cmdNext)
        self.line = QtWidgets.QFrame(wdgYearMonth)
        self.line.setFrameShape(QtWidgets.QFrame.VLine)
        self.line.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line.setObjectName("line")
        self.horizontalLayout.addWidget(self.line)
        self.cmdCurrent = QtWidgets.QPushButton(wdgYearMonth)
        self.cmdCurrent.setObjectName("cmdCurrent")
        self.horizontalLayout.addWidget(self.cmdCurrent)
        self.verticalLayout.addLayout(self.horizontalLayout)

        self.retranslateUi(wdgYearMonth)
        QtCore.QMetaObject.connectSlotsByName(wdgYearMonth)

    def retranslateUi(self, wdgYearMonth):
        _translate = QtCore.QCoreApplication.translate
        self.label.setText(_translate("wdgYearMonth", "Select a month and a year"))
        self.cmdPrevious.setToolTip(_translate("wdgYearMonth", "Previous month"))
        self.cmdPrevious.setText(_translate("wdgYearMonth", "<"))
        self.cmbMonth.setToolTip(_translate("wdgYearMonth", "Select a month"))
        self.cmbMonth.setItemText(0, _translate("wdgYearMonth", "January"))
        self.cmbMonth.setItemText(1, _translate("wdgYearMonth", "February"))
        self.cmbMonth.setItemText(2, _translate("wdgYearMonth", "March"))
        self.cmbMonth.setItemText(3, _translate("wdgYearMonth", "April"))
        self.cmbMonth.setItemText(4, _translate("wdgYearMonth", "May"))
        self.cmbMonth.setItemText(5, _translate("wdgYearMonth", "June"))
        self.cmbMonth.setItemText(6, _translate("wdgYearMonth", "July"))
        self.cmbMonth.setItemText(7, _translate("wdgYearMonth", "August"))
        self.cmbMonth.setItemText(8, _translate("wdgYearMonth", "September"))
        self.cmbMonth.setItemText(9, _translate("wdgYearMonth", "October"))
        self.cmbMonth.setItemText(10, _translate("wdgYearMonth", "November"))
        self.cmbMonth.setItemText(11, _translate("wdgYearMonth", "December"))
        self.cmbYear.setToolTip(_translate("wdgYearMonth", "Select a year"))
        self.cmdNext.setToolTip(_translate("wdgYearMonth", "Next month"))
        self.cmdNext.setText(_translate("wdgYearMonth", ">"))
        self.cmdCurrent.setToolTip(_translate("wdgYearMonth", "Current month"))
        self.cmdCurrent.setText(_translate("wdgYearMonth", "Current month"))





# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'xulpymoney/ui/frmQuotesIBM.ui'
#
# Created by: PyQt5 UI code generator 5.12.2
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_frmQuotesIBM(object):
    def setupUi(self, frmQuotesIBM):
        frmQuotesIBM.setObjectName("frmQuotesIBM")
        frmQuotesIBM.resize(529, 407)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/xulpymoney/add.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        frmQuotesIBM.setWindowIcon(icon)
        self.horizontalLayout_3 = QtWidgets.QHBoxLayout(frmQuotesIBM)
        self.horizontalLayout_3.setObjectName("horizontalLayout_3")
        self.verticalLayout = QtWidgets.QVBoxLayout()
        self.verticalLayout.setObjectName("verticalLayout")
        self.lblInvestment = QtWidgets.QLabel(frmQuotesIBM)
        font = QtGui.QFont()
        font.setPointSize(12)
        self.lblInvestment.setFont(font)
        self.lblInvestment.setStyleSheet("color: rgb(0, 192, 0);")
        self.lblInvestment.setAlignment(QtCore.Qt.AlignCenter)
        self.lblInvestment.setObjectName("lblInvestment")
        self.verticalLayout.addWidget(self.lblInvestment)
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_2.addItem(spacerItem)
        self.wdgDT = wdgDatetime(frmQuotesIBM)
        self.wdgDT.setFocusPolicy(QtCore.Qt.TabFocus)
        self.wdgDT.setObjectName("wdgDT")
        self.horizontalLayout_2.addWidget(self.wdgDT)
        spacerItem1 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_2.addItem(spacerItem1)
        self.verticalLayout.addLayout(self.horizontalLayout_2)
        self.chkNone = QtWidgets.QCheckBox(frmQuotesIBM)
        self.chkNone.setObjectName("chkNone")
        self.verticalLayout.addWidget(self.chkNone)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.label = QtWidgets.QLabel(frmQuotesIBM)
        self.label.setObjectName("label")
        self.horizontalLayout.addWidget(self.label)
        self.txtQuote = myQLineEdit(frmQuotesIBM)
        self.txtQuote.setObjectName("txtQuote")
        self.horizontalLayout.addWidget(self.txtQuote)
        self.verticalLayout.addLayout(self.horizontalLayout)
        self.chkCanBePurged = QtWidgets.QCheckBox(frmQuotesIBM)
        self.chkCanBePurged.setEnabled(False)
        self.chkCanBePurged.setChecked(True)
        self.chkCanBePurged.setObjectName("chkCanBePurged")
        self.verticalLayout.addWidget(self.chkCanBePurged)
        self.horizontalLayout_4 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_4.setObjectName("horizontalLayout_4")
        spacerItem2 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_4.addItem(spacerItem2)
        self.buttonbox = QtWidgets.QDialogButtonBox(frmQuotesIBM)
        self.buttonbox.setStandardButtons(QtWidgets.QDialogButtonBox.Cancel|QtWidgets.QDialogButtonBox.Ok)
        self.buttonbox.setObjectName("buttonbox")
        self.horizontalLayout_4.addWidget(self.buttonbox)
        spacerItem3 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_4.addItem(spacerItem3)
        self.verticalLayout.addLayout(self.horizontalLayout_4)
        self.horizontalLayout_3.addLayout(self.verticalLayout)

        self.retranslateUi(frmQuotesIBM)
        QtCore.QMetaObject.connectSlotsByName(frmQuotesIBM)
        frmQuotesIBM.setTabOrder(self.txtQuote, self.buttonbox)
        frmQuotesIBM.setTabOrder(self.buttonbox, self.chkNone)
        frmQuotesIBM.setTabOrder(self.chkNone, self.chkCanBePurged)
        frmQuotesIBM.setTabOrder(self.chkCanBePurged, self.wdgDT)

    def retranslateUi(self, frmQuotesIBM):
        _translate = QtCore.QCoreApplication.translate
        frmQuotesIBM.setWindowTitle(_translate("frmQuotesIBM", "Add a product price manually"))
        self.chkNone.setText(_translate("frmQuotesIBM", "Add a close session price"))
        self.label.setText(_translate("frmQuotesIBM", "Add a price"))
        self.chkCanBePurged.setText(_translate("frmQuotesIBM", "Can be purged?"))


from xulpymoney.ui.myqlineedit import myQLineEdit
from xulpymoney.ui.wdgDatetime import wdgDatetime
import xulpymoney.images.xulpymoney_rc



from PyQt5.QtWidgets import QDialog
from PyQt5.QtCore import QSize
from xulpymoney.ui.Ui_frmHelp import Ui_frmHelp

## Shows a Help dialog
class frmHelp(QDialog, Ui_frmHelp):
    def __init__(self,mem, parent = None, name = None, modal = False):
        QDialog.__init__(self, parent)
        self.mem=mem
        self.setupUi(self)
        self.resize(self.mem.settings.value("frmHelp/qdialog", QSize(800, 600)))

    ## Manage close event to save dialog size
    def closeEvent(self, event):
        self.mem.settings.setValue("frmHelp/qdialog", self.size())
        self.mem.settings.sync()



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'xulpymoney/ui/wdgOpportunitiesAdd.ui'
#
# Created by: PyQt5 UI code generator 5.12.2
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_wdgOpportunitiesAdd(object):
    def setupUi(self, wdgOpportunitiesAdd):
        wdgOpportunitiesAdd.setObjectName("wdgOpportunitiesAdd")
        wdgOpportunitiesAdd.resize(820, 298)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/xulpymoney/bank.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        wdgOpportunitiesAdd.setWindowIcon(icon)
        self.horizontalLayout = QtWidgets.QHBoxLayout(wdgOpportunitiesAdd)
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.verticalLayout = QtWidgets.QVBoxLayout()
        self.verticalLayout.setObjectName("verticalLayout")
        self.lbl = QtWidgets.QLabel(wdgOpportunitiesAdd)
        self.lbl.setMinimumSize(QtCore.QSize(800, 0))
        font = QtGui.QFont()
        font.setPointSize(14)
        font.setBold(True)
        font.setWeight(75)
        self.lbl.setFont(font)
        self.lbl.setText("")
        self.lbl.setAlignment(QtCore.Qt.AlignCenter)
        self.lbl.setObjectName("lbl")
        self.verticalLayout.addWidget(self.lbl)
        self.productSelector = wdgProductSelector(wdgOpportunitiesAdd)
        self.productSelector.setObjectName("productSelector")
        self.verticalLayout.addWidget(self.productSelector)
        self.formLayout = QtWidgets.QFormLayout()
        self.formLayout.setFieldGrowthPolicy(QtWidgets.QFormLayout.ExpandingFieldsGrow)
        self.formLayout.setObjectName("formLayout")
        self.deDate = QtWidgets.QDateEdit(wdgOpportunitiesAdd)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.deDate.sizePolicy().hasHeightForWidth())
        self.deDate.setSizePolicy(sizePolicy)
        self.deDate.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.deDate.setCalendarPopup(True)
        self.deDate.setObjectName("deDate")
        self.formLayout.setWidget(0, QtWidgets.QFormLayout.FieldRole, self.deDate)
        self.label_2 = QtWidgets.QLabel(wdgOpportunitiesAdd)
        self.label_2.setObjectName("label_2")
        self.formLayout.setWidget(0, QtWidgets.QFormLayout.LabelRole, self.label_2)
        self.txtEntry = myQLineEdit(wdgOpportunitiesAdd)
        self.txtEntry.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.txtEntry.setObjectName("txtEntry")
        self.formLayout.setWidget(1, QtWidgets.QFormLayout.FieldRole, self.txtEntry)
        self.label_5 = QtWidgets.QLabel(wdgOpportunitiesAdd)
        self.label_5.setObjectName("label_5")
        self.formLayout.setWidget(1, QtWidgets.QFormLayout.LabelRole, self.label_5)
        self.label = QtWidgets.QLabel(wdgOpportunitiesAdd)
        self.label.setObjectName("label")
        self.formLayout.setWidget(2, QtWidgets.QFormLayout.LabelRole, self.label)
        self.label_3 = QtWidgets.QLabel(wdgOpportunitiesAdd)
        self.label_3.setObjectName("label_3")
        self.formLayout.setWidget(3, QtWidgets.QFormLayout.LabelRole, self.label_3)
        self.txtStoploss = myQLineEdit(wdgOpportunitiesAdd)
        self.txtStoploss.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.txtStoploss.setObjectName("txtStoploss")
        self.formLayout.setWidget(3, QtWidgets.QFormLayout.FieldRole, self.txtStoploss)
        self.txtTarget = myQLineEdit(wdgOpportunitiesAdd)
        self.txtTarget.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.txtTarget.setObjectName("txtTarget")
        self.formLayout.setWidget(2, QtWidgets.QFormLayout.FieldRole, self.txtTarget)
        self.label_4 = QtWidgets.QLabel(wdgOpportunitiesAdd)
        self.label_4.setObjectName("label_4")
        self.formLayout.setWidget(4, QtWidgets.QFormLayout.LabelRole, self.label_4)
        self.cmbInvestmentTypePosition = QtWidgets.QComboBox(wdgOpportunitiesAdd)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.cmbInvestmentTypePosition.sizePolicy().hasHeightForWidth())
        self.cmbInvestmentTypePosition.setSizePolicy(sizePolicy)
        self.cmbInvestmentTypePosition.setSizeAdjustPolicy(QtWidgets.QComboBox.AdjustToContents)
        self.cmbInvestmentTypePosition.setObjectName("cmbInvestmentTypePosition")
        self.formLayout.setWidget(4, QtWidgets.QFormLayout.FieldRole, self.cmbInvestmentTypePosition)
        self.verticalLayout.addLayout(self.formLayout)
        self.buttonbox = QtWidgets.QDialogButtonBox(wdgOpportunitiesAdd)
        self.buttonbox.setStandardButtons(QtWidgets.QDialogButtonBox.Cancel|QtWidgets.QDialogButtonBox.Ok)
        self.buttonbox.setCenterButtons(True)
        self.buttonbox.setObjectName("buttonbox")
        self.verticalLayout.addWidget(self.buttonbox)
        self.horizontalLayout.addLayout(self.verticalLayout)

        self.retranslateUi(wdgOpportunitiesAdd)
        QtCore.QMetaObject.connectSlotsByName(wdgOpportunitiesAdd)
        wdgOpportunitiesAdd.setTabOrder(self.txtEntry, self.txtTarget)
        wdgOpportunitiesAdd.setTabOrder(self.txtTarget, self.txtStoploss)
        wdgOpportunitiesAdd.setTabOrder(self.txtStoploss, self.deDate)

    def retranslateUi(self, wdgOpportunitiesAdd):
        _translate = QtCore.QCoreApplication.translate
        self.deDate.setDisplayFormat(_translate("wdgOpportunitiesAdd", "yyyy/MM/dd"))
        self.label_2.setText(_translate("wdgOpportunitiesAdd", "Opportunity date"))
        self.txtEntry.setText(_translate("wdgOpportunitiesAdd", "0"))
        self.label_5.setText(_translate("wdgOpportunitiesAdd", "Entry"))
        self.label.setText(_translate("wdgOpportunitiesAdd", "Target"))
        self.label_3.setText(_translate("wdgOpportunitiesAdd", "Stop loss"))
        self.txtStoploss.setText(_translate("wdgOpportunitiesAdd", "0"))
        self.txtTarget.setText(_translate("wdgOpportunitiesAdd", "0"))
        self.label_4.setText(_translate("wdgOpportunitiesAdd", "Type of investment position"))


from xulpymoney.ui.myqlineedit import myQLineEdit
from xulpymoney.ui.wdgProductSelector import wdgProductSelector
import xulpymoney.images.xulpymoney_rc



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'xulpymoney/ui/wdgDividendsReport.ui'
#
# Created by: PyQt5 UI code generator 5.12.2
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_wdgDividendsReport(object):
    def setupUi(self, wdgDividendsReport):
        wdgDividendsReport.setObjectName("wdgDividendsReport")
        wdgDividendsReport.resize(1000, 531)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(wdgDividendsReport.sizePolicy().hasHeightForWidth())
        wdgDividendsReport.setSizePolicy(sizePolicy)
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout(wdgDividendsReport)
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.verticalLayout = QtWidgets.QVBoxLayout()
        self.verticalLayout.setObjectName("verticalLayout")
        self.lbl = QtWidgets.QLabel(wdgDividendsReport)
        font = QtGui.QFont()
        font.setPointSize(14)
        font.setBold(True)
        font.setWeight(75)
        self.lbl.setFont(font)
        self.lbl.setAlignment(QtCore.Qt.AlignCenter)
        self.lbl.setObjectName("lbl")
        self.verticalLayout.addWidget(self.lbl)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem)
        self.label = QtWidgets.QLabel(wdgDividendsReport)
        self.label.setAlignment(QtCore.Qt.AlignCenter)
        self.label.setObjectName("label")
        self.horizontalLayout.addWidget(self.label)
        self.spin = QtWidgets.QSpinBox(wdgDividendsReport)
        self.spin.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.spin.setMinimum(1)
        self.spin.setMaximum(3650)
        self.spin.setSingleStep(10)
        self.spin.setProperty("value", 90)
        self.spin.setObjectName("spin")
        self.horizontalLayout.addWidget(self.spin)
        self.cmd = QtWidgets.QPushButton(wdgDividendsReport)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/xulpymoney/alarm_clock.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.cmd.setIcon(icon)
        self.cmd.setObjectName("cmd")
        self.horizontalLayout.addWidget(self.cmd)
        spacerItem1 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem1)
        self.verticalLayout.addLayout(self.horizontalLayout)
        self.chkInactivas = QtWidgets.QCheckBox(wdgDividendsReport)
        self.chkInactivas.setObjectName("chkInactivas")
        self.verticalLayout.addWidget(self.chkInactivas)
        self.tblInvestments = myQTableWidget(wdgDividendsReport)
        self.tblInvestments.setContextMenuPolicy(QtCore.Qt.CustomContextMenu)
        self.tblInvestments.setEditTriggers(QtWidgets.QAbstractItemView.NoEditTriggers)
        self.tblInvestments.setAlternatingRowColors(True)
        self.tblInvestments.setSelectionMode(QtWidgets.QAbstractItemView.SingleSelection)
        self.tblInvestments.setSelectionBehavior(QtWidgets.QAbstractItemView.SelectRows)
        self.tblInvestments.setObjectName("tblInvestments")
        self.tblInvestments.setColumnCount(7)
        self.tblInvestments.setRowCount(0)
        item = QtWidgets.QTableWidgetItem()
        self.tblInvestments.setHorizontalHeaderItem(0, item)
        item = QtWidgets.QTableWidgetItem()
        self.tblInvestments.setHorizontalHeaderItem(1, item)
        item = QtWidgets.QTableWidgetItem()
        self.tblInvestments.setHorizontalHeaderItem(2, item)
        item = QtWidgets.QTableWidgetItem()
        self.tblInvestments.setHorizontalHeaderItem(3, item)
        item = QtWidgets.QTableWidgetItem()
        self.tblInvestments.setHorizontalHeaderItem(4, item)
        item = QtWidgets.QTableWidgetItem()
        self.tblInvestments.setHorizontalHeaderItem(5, item)
        item = QtWidgets.QTableWidgetItem()
        self.tblInvestments.setHorizontalHeaderItem(6, item)
        self.tblInvestments.verticalHeader().setVisible(False)
        self.verticalLayout.addWidget(self.tblInvestments)
        self.lblTotal = QtWidgets.QLabel(wdgDividendsReport)
        font = QtGui.QFont()
        font.setPointSize(8)
        font.setBold(True)
        font.setWeight(75)
        self.lblTotal.setFont(font)
        self.lblTotal.setAlignment(QtCore.Qt.AlignCenter)
        self.lblTotal.setObjectName("lblTotal")
        self.verticalLayout.addWidget(self.lblTotal)
        self.horizontalLayout_2.addLayout(self.verticalLayout)
        self.actionEstimationDPSEdit = QtWidgets.QAction(wdgDividendsReport)
        icon1 = QtGui.QIcon()
        icon1.addPixmap(QtGui.QPixmap(":/xulpymoney/coins.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.actionEstimationDPSEdit.setIcon(icon1)
        self.actionEstimationDPSEdit.setObjectName("actionEstimationDPSEdit")
        self.actionInvestmentReport = QtWidgets.QAction(wdgDividendsReport)
        icon2 = QtGui.QIcon()
        icon2.addPixmap(QtGui.QPixmap(":/xulpymoney/report2.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.actionInvestmentReport.setIcon(icon2)
        self.actionInvestmentReport.setObjectName("actionInvestmentReport")
        self.actionProductReport = QtWidgets.QAction(wdgDividendsReport)
        icon3 = QtGui.QIcon()
        icon3.addPixmap(QtGui.QPixmap(":/xulpymoney/kmplot.jpg"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.actionProductReport.setIcon(icon3)
        self.actionProductReport.setObjectName("actionProductReport")

        self.retranslateUi(wdgDividendsReport)
        QtCore.QMetaObject.connectSlotsByName(wdgDividendsReport)

    def retranslateUi(self, wdgDividendsReport):
        _translate = QtCore.QCoreApplication.translate
        self.lbl.setText(_translate("wdgDividendsReport", "Dividends Report"))
        self.label.setText(_translate("wdgDividendsReport", "Dividends per share are with a clock are outdated more than"))
        self.spin.setSuffix(_translate("wdgDividendsReport", " days"))
        self.chkInactivas.setText(_translate("wdgDividendsReport", "Show inactive investments"))
        item = self.tblInvestments.horizontalHeaderItem(0)
        item.setText(_translate("wdgDividendsReport", "Investment"))
        item = self.tblInvestments.horizontalHeaderItem(1)
        item.setText(_translate("wdgDividendsReport", "Bank"))
        item = self.tblInvestments.horizontalHeaderItem(2)
        item.setText(_translate("wdgDividendsReport", "Price"))
        item = self.tblInvestments.horizontalHeaderItem(3)
        item.setText(_translate("wdgDividendsReport", "DPS"))
        item = self.tblInvestments.horizontalHeaderItem(4)
        item.setText(_translate("wdgDividendsReport", "Shares"))
        item = self.tblInvestments.horizontalHeaderItem(5)
        item.setText(_translate("wdgDividendsReport", "Estimated"))
        item = self.tblInvestments.horizontalHeaderItem(6)
        item.setText(_translate("wdgDividendsReport", "% annual dividend"))
        self.actionEstimationDPSEdit.setText(_translate("wdgDividendsReport", "Update dividend per share"))
        self.actionEstimationDPSEdit.setToolTip(_translate("wdgDividendsReport", "Update dividend per share"))
        self.actionInvestmentReport.setText(_translate("wdgDividendsReport", "Investment report"))
        self.actionInvestmentReport.setToolTip(_translate("wdgDividendsReport", "Investment report"))
        self.actionProductReport.setText(_translate("wdgDividendsReport", "Product report"))
        self.actionProductReport.setToolTip(_translate("wdgDividendsReport", "Product report"))


from xulpymoney.ui.myqtablewidget import myQTableWidget
import xulpymoney.images.xulpymoney_rc
